from numpy import isfinite, any
from openopt.kernel.baseSolver import *
from sqlcp import sqlcp as SQLCP
from numpy.linalg import LinAlgError

class sqlcp(baseSolver):
    __name__ = 'sqlcp'
    __license__ = "MIT"
    __authors__ = "Enzo Michelangeli"
    __alg__ = "an SQP implementation"
    __optionalDataThatCanBeHandled__ = ['A', 'Aeq', 'b', 'beq', 'lb', 'ub']
    iterfcnConnected = True
    #_canHandleScipySparse = True
    QPsolver=None
    __info__ = '''SQP solver. Approximates f in x0 with paraboloid with same gradient and hessian,
    then finds its minimum with a quadratic solver (qlcp by default) and uses it as new point, 
    iterating till changes in x and/or f drop below given limits. 
    Requires the Hessian to be definite positive.
    The Hessian is initially approximated by its principal diagonal, and then
    updated at every step with the BFGS method.
    
    By default it uses QP solver qlcp (license: MIT), however, latter uses LCP solver LCPSolve, that is "free for education" for now. 
    You can use other QP solver via "oosolver('sqlcp', QPsolver='cvxopt_qp')" or any other, including converters.
    
    Copyright (c) 2010 Enzo Michelangeli and IT Vision Ltd
    '''

    def __init__(self): pass
    def __solver__(self, p):
        (A, b) = (p.A, p.b) if p.nb else (None, None)
        (Aeq, beq) = (p.Aeq, p.beq) if p.nbeq else (None, None)
        lb = p.lb if any(isfinite(p.lb)) else None
        ub = p.ub if any(isfinite(p.ub)) else None
        def callback(x):
            p.iterfcn(x)
            return True if p.istop else False
        try:
            SQLCP(p.f, p.x0, df=p.df, A=A , b=b, Aeq=Aeq, beq=beq, lb=p.lb, ub=p.ub, minstep=p.xtol, minfchg=1e-15, qpsolver=self.QPsolver, callback = callback)
        except LinAlgError:
            p.msg = 'linalg error, probably failed to invert Hesse matrix'
            p.istop = -100
            



"""
Example:
Solving nonsmooth problem
#K|x1| + |x2| -> min
#x0 = [10^4, 10]
x_opt = all-zeros
f_opt = 0
"""

from numpy import *
from openopt import NSP

K = 10**3

f = lambda x: abs(x[0]) + abs(x[1])*K + abs(x[2]) * K**2

x0 = [1000, 0.011, 0.01]

#OPTIONAL: user-supplied gradient/subgradient
df = lambda x: [sign(x[0]), sign(x[1])*K, sign(x[2]) * K**2]


#p.df = lambda x: 2*x
    
#p.plot = 0
#p.xlim = (inf,  5)
#p.ylim = (0, 5000000)
#p.checkdf()
solvers = ['r2', 'ipopt', 'algencan','ralg']
solvers = ['r2', 'algencan','ralg']
#solvers = ['ralg', 'r2']
solvers = ['r2', 'lincher']
solvers = ['ralg']
solvers = ['r2']
#solvers = ['scipy_slsqp']
#solvers = ['algencan']
#solvers = ['ipopt']
colors = ['r', 'b', 'k', 'g']
maxIter = 1000
for i, solver in enumerate(solvers):
    p = NSP(f, x0, df=df, xtol = 1e-11, ftol=1e-10,  maxIter = maxIter, maxTime=150)
    #p.checkdf()
    r = p.solve(solver, maxVectorNum=4, iprint=1, showLS=0, plot=0, color=colors[i], show=solver==solvers[-1]) # ralg is name of a solver
#for i, solver in enumerate(solvers):
#    p2 = NSP(f, r.xf, df=df, xtol = 1e-6, maxIter = 1200, maxTime=150, ftol=1e-6)
#    #p.checkdf()
#    r2 = p2.solve(solver, maxVectorNum=15, iprint=1, showLS=1, plot=0, color=colors[i], show=solver==solvers[-1]) # ralg is name of a solver
#print 'x_opt:\n', r.xf
print 'f_opt:', r.ff  # should print small positive number like 0.00056



from numpy import *
from numpy.linalg import norm

def project2box(x, lb, ub):
    X = atleast_1d(asfarray(x))
    lb, ub = atleast_1d(lb), atleast_1d(ub)
    projection = copy(X)
    ind = where(projection < lb)[0]
    projection[ind] = lb[ind]
    ind = where(projection > ub)[0]
    projection[ind] = ub[ind]
    distance = norm(X-projection)
    return projection, distance

def project2box_derivative(x, lb, ub):
    X = atleast_1d(asfarray(x))
    lb, ub = atleast_1d(lb), atleast_1d(ub)
    projection, distance = project2box(X, lb, ub)
    dX = zeros(X.shape)
    if distance == 0: return dX
    else:
        dX[where(X > ub)[0]] = 1.0
        dX[where(X < lb)[0]] = -1.0
        return dX

def project2ball(x, radius, center = 0.0):
    X = atleast_1d(asfarray(x))
    distance2center = norm(X-center)
    if distance2center  <= radius:
        return copy(x), 0.0
    else:
        projection = center + (X-center) * (radius / distance2center)
        distance = distance2center - radius
        return projection, distance

def project2ball_derivative(x, radius, center = 0.0):
    X = atleast_1d(asfarray(x))
    distance2center = norm(X-center)
    if distance2center  <= radius:
        return zeros(X.shape)
    else:
        distance = distance2center - radius
        return (X - center) / distance



from numpy import *
from openopt import *

coeff = 1e-7

f = lambda x: (x[0]-20)**2+(coeff * x[1] - 80)**2 # objFun
c = lambda x: (x[0]-14)**2-1 # non-lin ineq constraint(s) c(x) <= 0
# for the problem involved: f_opt =25, x_opt = [15.0, 8.0e9]

x0 = [-4,4]
# even modification of stop criteria can't help to achieve the desired solution:
someModifiedStopCriteria = {'gtol': 1e-15,  'ftol': 1e-13,  'xtol': 1e-13, 'maxIter': 1e3}

# using default diffInt = 1e-7 is inappropriate:
p = NLP(f, x0, c=c, iprint = 100, **someModifiedStopCriteria)
r = p.solve('ralg')
print r.ff,  r.xf #  will print something like "6424.9999886000014 [ 15.0000005   4.       ]"
"""
 for to improve the solution we will use
 changing either p.diffInt from default 1e-7 to [1e-7,  1]
 or p.scale from default None to [1,  1e-7]

 latter (using p.scale) is more recommended
 because it affects xtol for those solvers
 who use OO stop criteria
 (ralg, lincher, nsmm, nssolve and mb some others)
  xtol will be compared to scaled x shift:
 is || (x[k] - x[k-1]) * scale || < xtol

 You can define scale and diffInt as
 numpy arrays, matrices, Python lists, tuples
 """
p = NLP(f, x0, c=c, scale = [1,  coeff], iprint = 100, **someModifiedStopCriteria)
r = p.solve('ralg')
print r.ff,  r.xf # "24.999996490694787 [  1.50000004e+01   8.00004473e+09]" - much better
"""
Full Output:
-----------------------------------------------------
solver: ralg   problem: unnamed   goal: minimum
 iter    objFunVal    log10(maxResidual)   
    0  6.976e+03               2.51 
   51  6.425e+03              -6.10 
istop:  4 (|| F[k] - F[k-1] || < ftol)
Solver:   Time Elapsed = 0.16 	CPU Time Elapsed = 0.16
objFunValue: 6424.9999 (feasible, max constraint =  8e-07)
6424.999932 [ 15.0000004   4.       ]
-----------------------------------------------------
solver: ralg   problem: unnamed   goal: minimum
 iter    objFunVal    log10(maxResidual)   
    0  6.976e+03               2.51 
  100  4.419e+01              -5.99 
  200  2.504e+01              -6.10 
  300  2.503e+01              -6.10 
  400  2.503e+01              -6.10 
  500  2.503e+01              -6.10 
  506  2.500e+01              -6.91 
istop:  3 (|| X[k] - X[k-1] || < xtol)
Solver:   Time Elapsed = 1.59 	CPU Time Elapsed = 1.59
objFunValue: 25.000189 (feasible, max constraint =  1.23911e-07)
25.0001894297 [  1.50000001e+01   8.00137858e+08]
"""



"""
This is OpenOpt SDP example,
for the problem
http://openopt.org/images/1/12/SDP.png
"""

from numpy import mat
from openopt import SDP
S, d = {}, {}
S[0, 0] = mat('-7 -11; -11 3') # numpy array, array-like, CVXOPT matrix are allowed as well
S[0, 1] = mat('7 -18; -18 8')
S[0, 2] = mat('-2 -8; -8 1')

d[0] = mat('33, -9; -9, 26')

S[1, 0] = mat('-21 -11 0; -11 10 8; 0 8 5')
S[1, 1] = mat('0 10 16; 10 -10 -10; 16 -10 3')
S[1, 2] = mat('-5 2 -17; 2 -6 8; -17 -7 6')

d[1] = mat('14, 9, 40; 9, 91, 10; 40, 10, 15')

p = SDP([1, -1, 1], S = S, d = d)
# Also you can use A, b, Aeq, beq for linear matrix (in)equality constraints
# and lb, ub for box-bound constraints lb <= x <= ub
# see /examples/lp_1.py 

#r = p.solve('cvxopt_sdp', iprint = 0)
r = p.solve('dsdp', iprint = -1)

f_opt, x_opt = r.ff, r.xf
print('x_opt: %s' % x_opt)
print('f_opt: %s' % f_opt)
#x_opt: [-0.36766609  1.89832827 -0.88755043]
#f_opt: -3.15354478797



from NLOPT_AUX import NLOPT_AUX
from NLOPT_BASE import NLOPT_BASE
import nlopt

class slmvm1(NLOPT_BASE):
    __name__ = 'slmvm1'
    __alg__ = 'Shifted limited-memory variable-metric, rank 1'
    __authors__ = 'Prof. Ladislav Luksan'
    __isIterPointAlwaysFeasible__ = True
    
    __optionalDataThatCanBeHandled__ = ['lb', 'ub']
    
    def __init__(self): pass
    def __solver__(self, p):
        NLOPT_AUX(p, nlopt.LD_VAR1)



from MATLAB_BASE import MATLAB_BASE
#import numpy as np

class bintprog(MATLAB_BASE):
    __name__ = 'bintprog'
    __alg__ = ''
    __authors__ = ''
#    __isIterPointAlwaysFeasible__ = True
    
    __optionalDataThatCanBeHandled__ = ['lb', 'ub', 'A', 'Aeq', 'b', 'beq', 'intVars']
#    properTextOutput = True
    _canHandleScipySparse = True
    opts = ('Display', 'TolFun', 'MaxIter', 'MaxTime', 'TolXInteger')
    
    def __init__(self): pass
    def __solver__(self, p):
        #assert np.array_equiv(p.lb, 0) and np.array_equiv(p.ub, 1), 'bintprog handles only binary variables'
        
        x = MATLAB_BASE._Solve(self, p, p.f, p.A, p.b, p.Aeq, p.beq, p.x0)
        p.xk = p.xf = x.flatten()



from openopt.kernel.baseSolver import baseSolver
from openopt.kernel.nonOptMisc import Vstack, isspmatrix
from numpy.linalg import eig

#from nonOptMisc import Hstack

class numpy_eig(baseSolver):
    __name__ = 'numpy_eig'
    __license__ = "BSD"
    __authors__ = ''
    __alg__ = ''
    __info__ = """    """

    __optionalDataThatCanBeHandled__ = []
    _canHandleScipySparse = True
    
    #def __init__(self): pass

    def __solver__(self, p):
        A = p.C
        if isspmatrix(A):
            p.warn('numpy.linalg.eig cannot handle sparse matrices, cast to dense will be performed')
            A = A.A
        #M = p.M
        
        if p._goal != 'all':
            p.err('numpy_eig cannot handle the goal "%s" yet' % p._goal)
        
        eigenvalues, eigenvectors = eig(A)
        p.xf = p.xk = Vstack((eigenvalues, eigenvectors))
        p.eigenvalues = eigenvalues
        p.eigenvectors = eigenvectors
        p.ff = 0



from numpy import empty, logical_and, logical_not, take, zeros, isfinite, any, \
asarray, ndarray, bool_#where
from interalgT import truncateByPlane, getTruncatedArrays, splitDomainForDiscreteVariable#adjustDiscreteVarBounds
import numpy as np

# for PyPy
from openopt.kernel.nonOptMisc import where

hasPoint = lambda y, e, point:\
    True if y.size != 0 and any([(np.all(y[i]<=point) and np.all(e[i]>=point)) for i in range(y.shape[0])]) else False
pointInd = lambda y, e, point:\
    where([(np.all(y[i]<=point) and np.all(e[i]>=point)) for i in range(y.shape[0])])[0].tolist()
    
def processConstraints(C0, y, e, _s, p, dataType):
    #P = np.array([  7.64673334e-01,    4.35551807e-01,    5.93869991e+02,   5.00000000e+00])
#    P = np.array([-0.63521194458007812, -0.3106536865234375, 0.0905609130859375, 0.001522064208984375, -0.69999999999999996, -0.99993896484375, 0.90000152587890625, 1.0, 4.0])

#    print('c-1', p.iter, hasPoint(y, e, P), pointInd(y, e, P))
    n = p.n
    m = y.shape[0]
    indT = empty(m, bool)
    indT.fill(False)
#    isSNLE = p.probType in ('NLSP', 'SNLE')
    
    for i in range(p.nb):
        y, e, indT, ind_trunc = truncateByPlane(y, e, indT, p.A[i], p.b[i]+p.contol)
        if ind_trunc is not True:
            _s = _s[ind_trunc]
    for i in range(p.nbeq):
        # TODO: handle it via one func
        y, e, indT, ind_trunc = truncateByPlane(y, e, indT, p.Aeq[i], p.beq[i]+p.contol)
        if ind_trunc is not True:
            _s = _s[ind_trunc]
        y, e, indT, ind_trunc = truncateByPlane(y, e, indT, -p.Aeq[i], -p.beq[i]+p.contol)
        if ind_trunc is not True:
            _s = _s[ind_trunc]
   
    DefiniteRange = True
    m = y.shape[0]
    nlh = zeros((m, 2*n))
    nlh_0 = zeros(m)
#    if len(p._discreteVarsNumList):
#        y, e, _s, indT = adjustDiscreteVarBounds(y, e, _s, indT, p)
#        y, e = adjustDiscreteVarBounds(y, e, p)
        #y, e, trunc_ind = adjustDiscreteVarBounds(y, e, p)
#        y, e, indT, _s, nlh, nlh_0 = \
#        getTruncatedArrays2(trunc_ind, y, e, indT, _s, nlh, nlh_0)

    m = y.shape[0]
    
    # TODO: probably remove it
    fullOutput = False#isSNLE and not p.hasLogicalConstraints
    
    residual = zeros((m, 2*n)) if fullOutput else None
    residual_0 = zeros(m) if fullOutput else None
    
    for itn, (c, f, lb, ub, tol) in enumerate(C0):
#        assert np.all(y == np.floor(y))
#        assert np.all(e == np.floor(e))
#        print ('c_1', itn, c.dep, hasPoint(y, e, P))
        m = y.shape[0] # is changed in the cycle
        if m == 0: 
            return y.reshape(0, n), e.reshape(0, n), nlh.reshape(0, 2*n), residual, True, False, _s
            #return y.reshape(0, n), e.reshape(0, n), nlh.reshape(0, 2*n), residual.reshape(0, 2*n), True, False, _s
        assert nlh.shape[0] == y.shape[0]
        
        if fullOutput:
            (T0, Res0), (res, R_res), DefiniteRange2 = c.nlh(y, e, p, dataType, fullOutput = True)
            residual_0 += Res0
        else:
            # may be logical constraint and doesn't have kw fullOutput at all
            T0, res, DefiniteRange2 = c.nlh(y, e, p, dataType)
        DefiniteRange = logical_and(DefiniteRange, DefiniteRange2)
        
        assert T0.ndim <= 1, 'unimplemented yet'

        nlh_0 += T0
        assert nlh.shape[0] == m
        # TODO: rework it for case len(p._freeVarsList) >> 1

        for v, tmp in res.items():
            j = p._freeVarsDict.get(v)
            nlh[:, n+j] += tmp[:, tmp.shape[1]/2:].flatten() - T0
            nlh[:, j] += tmp[:, :tmp.shape[1]/2].flatten() - T0
            if fullOutput:
                Tmp = R_res[v]
                residual[:, n+j] += Tmp[:, Tmp.shape[1]/2:].flatten() - Res0
                residual[:, j] += Tmp[:, :Tmp.shape[1]/2].flatten() - Res0
                    
        assert nlh.shape[0] == m
        ind = where(logical_and(any(isfinite(nlh), 1), isfinite(nlh_0)))[0]
        lj = ind.size
        if lj != m:
            assert nlh.shape[0] == y.shape[0]
            y = take(y, ind, axis=0, out=y[:lj])
            e = take(e, ind, axis=0, out=e[:lj])
            nlh = take(nlh, ind, axis=0, out=nlh[:lj])
            nlh_0 = nlh_0[ind]
            assert nlh.shape[0] == y.shape[0]
#            residual = take(residual, ind, axis=0, out=residual[:lj])
            indT = indT[ind]
            _s = _s[ind]
            if fullOutput:
                residual_0 = residual_0[ind]
                residual = take(residual, ind, axis=0, out=residual[:lj])
            if asarray(DefiniteRange).size != 1: 
                DefiniteRange = take(DefiniteRange, ind, axis=0, out=DefiniteRange[:lj])
#            print ('c_2', itn, c.dep, hasPoint(y, e, P))
        assert nlh.shape[0] == y.shape[0]


        ind = logical_not(isfinite(nlh)) #& False
        if any(ind):
            indT[any(ind, 1)] = True
            
            ind_l,  ind_u = ind[:, :ind.shape[1]/2], ind[:, ind.shape[1]/2:]
#            ind_ = logical_or(logical_not(ind_l), logical_not(ind_u))
            tmp_l, tmp_u = 0.5 * (y[ind_l] + e[ind_l]), 0.5 * (y[ind_u] + e[ind_u])
            
            # TODO: improve it, don't copy for continuous variables
            if len(p._discreteVarsNumList):
                r10 = np.array(p._discreteVarsNumList, int)
                L, U = y[:, r10].copy(), e[:, r10].copy()
                
            y[ind_l], e[ind_u] = tmp_l, tmp_u
            
            if len(p._discreteVarsNumList):
                y[:, r10], e[:, r10] = L, U
            
            for i in p._discreteVarsNumList:
                v = p._freeVarsList[i]
                
                ind_l1 = where(ind_l[:, i])[0]
                if ind_l1.size:
                    mid1, mid2 = splitDomainForDiscreteVariable(y[ind_l1, i], e[ind_l1, i], v)
#                    y[ind_l1, i] = L[ind_l1, n+i]
                    y[ind_l1, i] = mid2
#                    y[ind_l1, n+i] = mid2
                    
                ind_u1 = where(ind_u[:, i])[0]
                if ind_u1.size:
                    mid1, mid2 = splitDomainForDiscreteVariable(y[ind_u1, i], e[ind_u1, i], v)
                    e[ind_u1, i] = mid1
#                    e[ind_u1, n+i] = U[ind_u1, i]
#                    e[ind_u1, i] = mid1

            # TODO: mb lock is required for parallel computations
            
            nlh_l, nlh_u = nlh[:, nlh.shape[1]/2:], nlh[:, :nlh.shape[1]/2]
            
            # inplace operations are performed in the cycle
            if 1:
                nlh_l[ind_u], nlh_u[ind_l] = nlh_u[ind_u].copy(), nlh_l[ind_l].copy()   
            else:
                ind_Tmp = logical_and(ind_u, logical_not(ind_l))
                nlh_l[ind_Tmp] = nlh_u[ind_Tmp].copy()
                ind_Tmp = logical_and(ind_l, logical_not(ind_u))
                nlh_u[ind_Tmp] = nlh_l[ind_Tmp].copy()
            
            
            if fullOutput:
                residual_l, residual_u = residual[:, residual.shape[1]/2:], residual[:, :residual.shape[1]/2]
                residual_l[ind_u], residual_u[ind_l] = residual_u[ind_u].copy(), residual_l[ind_l].copy()   
#            print ('c_3', itn, c.dep, hasPoint(y, e, P))

    if nlh.size != 0:
        if DefiniteRange is False:
            nlh_0 += 1e-300
        elif type(DefiniteRange) == ndarray and not all(DefiniteRange):
            nlh_0[logical_not(DefiniteRange)] += 1e-300
        else:
            assert type(DefiniteRange) in (bool, bool_, ndarray)
    # !! matrix - vector
    nlh += nlh_0.reshape(-1, 1)
    
    if fullOutput:
        # !! matrix - vector
        residual += residual_0.reshape(-1, 1)
        residual[residual_0>=1e300] = 1e300
    
    #print('c2', p.iter, hasPoint(y, e, P), pointInd(y, e, P))
    return y, e, nlh, residual, DefiniteRange, indT, _s

hasPoint = lambda y, e, point:\
    True if y.size != 0 and any([(all(y[i]<=point) and all(e[i]>=point)) for i in range(y.shape[0])]) else False
pointInd = lambda y, e, point:\
    where([(all(y[i]<=point) and all(e[i]>=point)) for i in range(y.shape[0])])[0].tolist()
    

def getTruncatedArrays2(ind, y, e, indT, _s, nlh, nlh_0, ind_l = None, ind_u = None):
    # TODO: rework it when numpy will have appropriate inplace function
    y, e, indT, _s = getTruncatedArrays(ind, y, e, indT, _s)
    s = ind.size
    nlh_0 = nlh_0[ind]
    nlh = take(nlh, ind, axis=0, out=nlh[:s])
    
    if ind_l is None:
        return y, e, indT, _s, nlh, nlh_0

    ind_l = take(ind_l, ind, axis=0, out=ind_l[:s])
    ind_u = take(ind_u, ind, axis=0, out=ind_u[:s])
    return y, e, indT, _s, nlh, nlh_0, ind_l, ind_u
    

'''
            if len(p._discreteVarsNumList):
                tmp_l, tmp_u = adjustDiscreteVarBounds(tmp_l, tmp_u, p)
                if tmp_l.ndim > 1:
                    # shouldn't reduce y,e shape here, so output values doesn't matter
#                    l1 = len(_s)
                    assert nlh.shape[0] == y.shape[0]
#                    tmp_l, tmp_u, _s2, indT2 = \
#                    adjustDiscreteVarBounds(tmp_l, tmp_u, _s, indT, p)
                    #adjustDiscreteVarBounds(tmp_l, tmp_u, p, (indT, _s, nlh, nlh_0))
                    
                    
#                    tmp_l, tmp_u = adjustDiscreteVarBounds(tmp_l, tmp_u, p)
                    
                    
                    #tmp_l, tmp_u, trunc_ind = adjustDiscreteVarBounds(tmp_l, tmp_u, p)
#                    tmp_l, tmp_u, indT, _s, nlh, nlh_0, ind_l, ind_u  = \
#                    getTruncatedArrays2(trunc_ind, tmp_l, tmp_u, indT, _s, nlh, nlh_0)
#                    y[ind_l], e[ind_u] = tmp_l, tmp_u
                    
                    assert nlh.shape[0] == y.shape[0]
#                    l2 = len(trunc_ind)
#                    if l2 != 0:
#                        print('Warning: possible bug in interalg constraints processing, inform developers')
                else:
                    pass
#                    y, e = adjustDiscreteVarBounds(y, e, p)
                    #y, e, trunc_ind = adjustDiscreteVarBounds(y, e, p)
#                    y, e, indT, _s, nlh, nlh_0, ind_l, ind_u = \
#                    getTruncatedArrays2(trunc_ind, y, e, indT, _s, nlh, nlh_0, ind_l, ind_u)
                    #y, e, _s, indT = adjustDiscreteVarBounds(y, e, _s, indT, p)
'''



from openopt import NLP
from numpy import cos, arange, ones, asarray, abs, zeros, sqrt, asscalar
from pylab import legend, show, plot, subplot, xlabel, subplots_adjust
from string import rjust, ljust, expandtabs
N = 15
M = 5
f = lambda x: -(abs(x-M) ** 1.5).sum()

x0 = cos(arange(N))

#c = lambda x: [2* x[0] **4-32, x[1]**2+x[2]**2 - 8]
global cc1, cc2, cc3

def c1(x):
    global cc1
    cc1 += 1
    return 2* x[0] **4-32



def c2(x):
    global cc2
    cc2 += 1
    return x[1]**2+x[2]**2 - 8


def c3(x):
    global cc3
    cc3 += 1
    return x[1]**2+x[2]**2 + x[3]**2  - 35


c = [c1, c2, c3]

h1 = lambda x: 1e1*(x[-1]-1)**4
h2 = lambda x: (x[-2]-1.5)**4
h = (h1, h2)

lb = -6*ones(N)
ub = 6*ones(N)
lb[3] = 5.5
ub[4] = 4.5


colors = ['b', 'k', 'y', 'r', 'g']
#############
#solvers = ['ralg','scipy_cobyla', 'lincher']
solvers = ['ralg', 'scipy_cobyla', 'lincher','ipopt','algencan' ]
solvers = ['ralg', 'ralg3', 'ralg5']
solvers = ['ralg',  'scipy_cobyla']
#solvers = ['ipopt']
solvers = ['ralg', 'ipopt']
solvers = ['ralg']
solvers = ['scipy_slsqp']
solvers = ['ralg']
#############
colors = colors[:len(solvers)]

lines, results = [], {}
for j in range(len(solvers)):
    cc1,  cc2,  cc3 = 0, 0, 0
    solver = solvers[j]
    color = colors[j]
    p = NLP(f, x0, c=c, h=h, lb = lb, ub = ub, ftol = 1e-6, maxFunEvals = 1e7, maxIter = 1220, plot = 1, color = color, iprint = 0, legend = [solvers[j]], show= False, xlabel='time', goal='maximum', name='nlp3')
    if solver == 'algencan':
        p.gtol = 1e-1
    elif solver == 'ralg':
        p.debug = 1

    r = p.solve(solver, debug=1)
    print 'c1 evals:', cc1, 'c2 evals:', cc2, 'c3 evals:', cc3
    results[solver] = (r.ff, p.getMaxResidual(r.xf), r.elapsed['solver_time'], r.elapsed['solver_cputime'], r.evals['f'], r.evals['c'], r.evals['h'])
    subplot(2,1,1)
    F0 = asscalar(p.f(p.x0))
    lines.append(plot([0, 1e-15], [F0, F0], color= colors[j]))

for i in range(2):
    subplot(2,1,i+1)
    legend(lines, solvers)

subplots_adjust(bottom=0.2, hspace=0.3)

xl = ['Solver                              f_opt     MaxConstr   Time   CPUTime  fEvals  cEvals  hEvals']
for i in range(len(results)):
    xl.append((expandtabs(ljust(solvers[i], 16)+' \t', 15)+'%0.2f'% (results[solvers[i]][0]) + '        %0.1e' % (results[solvers[i]][1]) + ('      %0.2f'% (results[solvers[i]][2])) + '    %0.2f      '% (results[solvers[i]][3]) + str(results[solvers[i]][4]) + '   ' + rjust(str(results[solvers[i]][5]), 5) + expandtabs('\t' +str(results[solvers[i]][6]),8)))

xl = '\n'.join(xl)
subplot(2,1,1)
xlabel('Time elapsed (without graphic output), sec')

from pylab import *
subplot(2,1,2)
xlabel(xl)

show()




PythonAny = any
from numpy import asscalar, copy, asarray, isfinite, ndarray, array, where, any
from fdmisc import formDictOfFixedFuncs, linear_render, formDictOfRedirectedFuncs


def fdConstraint(p, c, StartPointVars, areFixed, oovD, A, b, Aeq, beq, Z, D_kwargs, LB, UB, inplaceLinearRender):
        #import FuncDesigner as fd
        from FuncDesigner.constraints import SmoothFDConstraint
        from FuncDesigner.BooleanOOFun import BooleanOOFun
        from FuncDesigner import _Stochastic
        if not isinstance(c, SmoothFDConstraint) and isinstance(c, BooleanOOFun): 
            p.hasLogicalConstraints = True
            #continue
        probtol = p.contol
        f, tol = c.oofun, c.tol
        _lb, _ub = c.lb, c.ub
        Name = f.name
        
        dep = f.Dep
        
        isFixed = areFixed(dep)

        if f.is_oovar and isFixed:  
            if p._x0 is None or f not in p._x0: 
                p.err('your problem has fixed oovar '+ Name + ' but no value for the one in start point is provided')
            
            if isinstance(p._x0[f], _Stochastic):
                p.err('''
                Bounds %g <= %s <= %g on distribution "%s":
                box bounds and other direct constraints 
                cannot be directly applied on stochastic variables,
                you should use functions like mean, std, var, P.''' % (c.lb, f.name, c.ub, p._x0[f])
                )
            
            # TODO: mb use c.tol / p.contol?
            if not c.lb <= p._x0[f] <= c.ub:
                s ='''fixed variable "%s" has infeasible box constraint in start point
                (%g <= %g <= %g) 
                thus the problem is infeasible
                ''' % (f.name, c.lb, p._x0[f], c.ub)
                p.err(s)
            
            return True
        
        if not dep.issubset(StartPointVars):
            p.err('your start point has no enough variables to define constraint ' + c.name)

        if tol < 0:
            if any(_lb  == _ub):
                p.err("You can't use negative tolerance for the equality constraint " + c.name)
            elif any(_lb - tol >= _ub + tol):
                p.err("You can't use negative tolerance for so small gap in constraint" + c.name)

            Shift = (1.0+1e-13)*probtol 
            #######################
            # not inplace modification!!!!!!!!!!!!!
            _lb = _lb + Shift
            _ub = _ub - Shift
            #######################
        
        if tol != 0: p.useScaledResidualOutput = True
        
        # TODO: omit it for interalg
        if tol not in (0, probtol, -probtol):
            scaleFactor = abs(probtol / tol)
            
            f *= scaleFactor
            #c.oofun = f#c.oofun * scaleFactor
            _lb, _ub = _lb * scaleFactor, _ub * scaleFactor
            Contol = tol
            Contol2 = Contol * scaleFactor
        else:
            Contol = asscalar(copy(probtol))
            Contol2 = Contol 
            
        if isFixed:
            # TODO: get rid of p.contol, use separate contols for each constraint
            
            if not c(p._x0, tol=Contol2):
                s = """'constraint "%s" with all-fixed optimization variables it depends on is infeasible in start point, 
                hence the problem is infeasible, maybe you should change start point'""" % c.name
                p.err(s)
            return True

        from FuncDesigner import broadcast
        hasFixedVariables = len(p.fixedVarsSet)
#        for j in range(2):
        if hasFixedVariables:
            broadcast(formDictOfFixedFuncs, f, p.useAttachedConstraints, p)
        if p._isStochastic:
            broadcast(formDictOfRedirectedFuncs, f, p.useAttachedConstraints, p)
            if 0 and hasFixedVariables:
                broadcast(formDictOfFixedFuncs, f, p.useAttachedConstraints, p.dictOfFixedFuncs, areFixed, p._x0)
                #TODO: is it ever encountered?
                #broadcast(formDictOfRedirectedFuncs, f, p.useAttachedConstraints, p)

        f_order = f.getOrder(p.freeVarsSet, p.fixedVarsSet, fixedVarsScheduleID = p._FDVarsID)
#        print('c_order:', f_order)
        
        if p.probType in ['LP', 'MILP', 'LLSP', 'LLAVP'] and f_order > 1:
            p.err('for LP/MILP/LLSP/LLAVP all constraints have to be linear, while ' + f.name + ' is not')
        
        if not f.is_oovar and f_order < 2:
            D_kwargs2 = D_kwargs.copy()
            if inplaceLinearRender:
                # interalg only
                D_kwargs2['useSparse'] = False
            D = f.D(Z, **D_kwargs2)
            if inplaceLinearRender:
                # interalg only
                if PythonAny(asarray(val).size > 1 for val in D.values()):
                    p.err('currently interalg can handle only FuncDesigner.oovars(n), not FuncDesigner.oovar() with size > 1')
                f = linear_render(f, D, Z)
#        else:
#            D = {}
        
        # TODO: simplify condition of box-bounded oovar detection
        if f.is_oovar:
            inds = oovD[f]
            f_size = inds[1] - inds[0]

            if any(isfinite(_lb)):
                if _lb.size not in (f_size, 1): 
                    p.err('incorrect size of lower box-bound constraint for %s: 1 or %d expected, %d obtained' % (Name, f_size, _lb.size))
                    
                # for PyPy compatibility
                if type(_lb) == ndarray and _lb.size == 1:
                    _lb = _lb.item()
                
                val = array(f_size*[_lb] if type(_lb) == ndarray and _lb.size < f_size else _lb)
                LB[f] = val if f not in LB else where(val > LB[f], val, LB[f])

            if any(isfinite(_ub)):
                if _ub.size not in (f_size, 1): 
                    p.err('incorrect size of upper box-bound constraint for %s: 1 or %d expected, %d obtained' % (Name, f_size, _ub.size))
                
                # for PyPy compatibility
                if type(_ub) == ndarray and _ub.size == 1:
                    _ub = _ub.item()
                    
                val = array(f_size*[_ub] if type(_ub) == ndarray and _ub.size < f_size else _ub)
                UB[f] = val if f not in UB else where(val < UB[f], val, UB[f])
                    
        elif _lb == _ub:
            if f_order < 2:
                Aeq.append(p._pointDerivative2array(D))      
                beq.append(-f(Z)+_lb)
            elif p.h is None: p.h = [f-_lb]
            else: p.h.append(f-_lb)
        elif isfinite(_ub):
            if f_order < 2:
                A.append(p._pointDerivative2array(D))                       
                b.append(-f(Z)+_ub)
            elif p.c is None: p.c = [f - _ub]
            else: p.c.append(f - _ub)
        elif isfinite(_lb):
            if f_order < 2:
                A.append(-p._pointDerivative2array(D))                       
                b.append(f(Z) - _lb)                        
            elif p.c is None: p.c = [- f + _lb]
            else: p.c.append(- f + _lb)
        else:
            p.err('inform OpenOpt developers of the bug')
            
        if not f.is_oovar:
            Contol = max((0, Contol2))
            # TODO: handle it more properly, especially  for lb, ub of array type
            # FIXME: name of f0 vs f
#            p._FD.nonBoxConsWithTolShift.append((f0, lb_0 - Contol, ub_0 + Contol))
#            p._FD.nonBoxCons.append((f0, lb_0, ub_0, Contol))
            p._FD.nonBoxConsWithTolShift.append((c, f, _lb - Contol, _ub + Contol))
            p._FD.nonBoxCons.append((c, f, _lb, _ub, Contol))
#            if tol not in (0, probtol, -probtol):
#                print('!', f, _lb, _ub, Contol)
        return False






from openopt import NLP

from numpy import cos, arange, ones, asarray, zeros, mat, array, sin, cos, sign, abs, inf
N = 1500
K = 50
# 1st arg - objective function
# 2nd arg - x0

p = NLP(lambda x: (abs(x-5)).sum(), 8*cos(arange(N)), iprint = 50, maxIter = 1e3)

# f(x) gradient (optional):
p.df = lambda x: sign(x-5)

p.lb = 5*ones(N) + sin(arange(N)) - 0.1
p.ub = 5*ones(N) + sin(arange(N)) + 0.1
p.lb[:N/4] = -inf
p.ub[3*N/4:] = inf

#p.ub[4] = 4
#p.lb[5], p.ub[5] = 8, 15

#A = zeros((K, N))
#b = zeros(K)
#for i in xrange(K):
#    A[i] = 1+cos(i+arange(N))
#    b[i] = sin(i)
#p.A = A
#p.b = b

#p.Aeq = zeros(p.n)
#p.Aeq[100:102] = 1
#p.beq = 11

p.contol = 1e-6
p.plot = 1
p.maxFunEvals = 1e7
p.name = 'nlp_4'
p.debug=1
solver = 'ralg'
solver = 'gsubg'
#solver = 'scipy_cobyla'
solver = 'algencan'
#solver = 'ipopt'

r = p.solve(solver, xlabel = 'time', fTol = 10, debug=0, maxIter = 5500, plot=0, maxTime=1000, ftol = 1e-8, xtol = 1e-6, iprint=1, showLS=0, showFeas=0, show_hs=0)




from openopt import NLP
from numpy import cos, arange, ones, asarray, zeros, mat, array

N = 50
# objfunc:
# (x0-1)^4 + (x2-1)^4 + ... +(x49-1)^4 -> min (N=nVars=50)
f = lambda x : ((x-1)**4).sum()
x0 = cos(arange(N))
p = NLP(f, x0, maxIter = 1e3, maxFunEvals = 1e5)

# f(x) gradient (optional):
p.df = lambda x: 4*(x-1)**3


# lb<= x <= ub:
# x4 <= -2.5
# 3.5 <= x5 <= 4.5
# all other: lb = -5, ub = +15
p.lb = -5*ones(N)
p.ub = 15*ones(N)
p.ub[4] = -2.5
p.lb[5], p.ub[5] = 3.5, 4.5



# Ax <= b
# x0+...+xN>= 1.1*N
# x9 + x19 <= 1.5
# x10+x11 >= 1.6
p.A = zeros((3, N))
p.A[0, 9] = 1
p.A[0, 19] = 1
p.A[1, 10:12] = -1
p.A[2] = -ones(N)
p.b = [1.5, -1.6, -1.1*N]
# you can use any types of A, Aeq, b, beq:
# Python list, numpy.array, numpy.matrix, even Python touple
# so p.b = array([1.5, -1.6, -825]) or p.b = (1.5, -1.6, -825) are valid as well


# Aeq x = beq
# x20+x21 = 2.5
p.Aeq = zeros(N)
p.Aeq[20:22] = 1
p.beq = 2.5


# non-linear inequality constraints c(x) <= 0
# 2*x0^4 <= 1/32
# x1^2+x2^2 <= 1/8
# x25^2 +x25*x35 + x35^2<= 2.5

p.c = lambda x: [2* x[0] **4-1./32, x[1]**2+x[2]**2 - 1./8, x[25]**2 + x[35]**2 + x[25]*x[35] -2.5]
# other valid c:
# p.c = [lambda x: c1(x), lambda x : c2(x), lambda x : c3(x)]
# p.c = (lambda x: c1(x), lambda x : c2(x), lambda x : c3(x))
# p.c = lambda x: numpy.array(c1(x), c2(x), c3(x))
# def c(x):
#      return c1(x), c2(x), c3(x)
# p.c = c


# dc(x)/dx: non-lin ineq constraints gradients (optional):
def DC(x):
    r = zeros((3, N))
    r[0,0] = 2 * 4 * x[0]**3
    r[1,1] = 2 * x[1]
    r[1,2] = 2 * x[2]
    r[2,25] = 2*x[25] + x[35]
    r[2,35] = 2*x[35] + x[25]
    return r
p.dc = DC

# non-linear equality constraints h(x) = 0
# 1e6*(x[last]-1)**4 = 0
# (x[last-1]-1.5)**4 = 0

p.h = lambda x: (1e4*(x[-1]-1)**4, (x[-2]-1.5)**4)
# dh(x)/dx: non-lin eq constraints gradients (optional):
def DH(x):
    r = zeros((2, p.n))
    r[0, -1] = 1e4*4 * (x[-1]-1)**3
    r[1, -2] = 4 * (x[-2]-1.5)**3
    return r
p.dh = DH

p.contol = 1e-3 # required constraints tolerance, default for NLP is 1e-6

# for ALGENCAN solver gtol is the only one stop criterium connected to openopt
# (except maxfun, maxiter)
# Note that in ALGENCAN gtol means norm of projected gradient of  the Augmented Lagrangian
# so it should be something like 1e-3...1e-5
p.gtol = 1e-5 # gradient stop criterium (default for NLP is 1e-6)


# see also: help(NLP) -> maxTime, maxCPUTime, ftol and xtol
# that are connected to / used in lincher and some other solvers

# optional: check of user-supplied derivatives
p.checkdf()
p.checkdc()
p.checkdh()

# last but not least:
# please don't forget,
# Python indexing starts from ZERO!!

p.plot = 0
p.iprint = 0
p.df_iter = 4
p.maxTime = 4000
p.debug=1
#r = p.solve('algencan')

r = p.solve('ralg')
#r = p.solve('lincher')

"""
typical output:
OpenOpt checks user-supplied gradient df (size: (50,))
according to:
prob.diffInt = 1e-07
prob.check.maxViolation = 1e-05
max(abs(df_user - df_numerical)) = 2.50111104094e-06
(is registered in df number 41)
sum(abs(df_user - df_numerical)) = 4.45203815948e-05
========================
OpenOpt checks user-supplied gradient dc (size: (50, 3))
according to:
prob.diffInt = 1e-07
prob.check.maxViolation = 1e-05
max(abs(dc_user - dc_numerical)) = 1.20371180401e-06
(is registered in dc number 0)
sum(abs(dc_user - dc_numerical)) = 1.60141862837e-06
========================
OpenOpt checks user-supplied gradient dh (size: (50, 2))
according to:
prob.diffInt = 1e-07
prob.check.maxViolation = 1e-05
dh num   i,j:dh[i]/dx[j]   user-supplied     numerical         difference
     98            49 / 0         -1.369e+04     -1.369e+04     -2.941e-03
max(abs(dh_user - dh_numerical)) = 0.00294061290697
(is registered in dh number 98)
sum(abs(dh_user - dh_numerical)) = 0.00294343472179
========================
starting solver ALGENCAN (GPL  license)  with problem  unnamed
solver ALGENCAN has finished solving the problem unnamed
istop:  1000
Solver:   Time elapsed = 0.34   CPU Time Elapsed = 0.34
objFunValue: 190.041570332 (feasible, max constraint =  0.000677961)
"""



__docformat__ = "restructuredtext en"
#from numpy import *
import numpy as np
#TODO: add stopcases -10,-11,-12, -13
SMALL_DF = 2
SMALL_DELTA_X = 3
SMALL_DELTA_F = 4
FVAL_IS_ENOUGH = 10
MAX_NON_SUCCESS = 11
USER_DEMAND_STOP = 80
BUTTON_ENOUGH_HAS_BEEN_PRESSED = 88
SOLVED_WITH_UNIMPLEMENTED_OR_UNKNOWN_REASON = 1000



UNDEFINED = 0

IS_NAN_IN_X = -4
IS_LINE_SEARCH_FAILED = -5
IS_MAX_ITER_REACHED = -7
IS_MAX_CPU_TIME_REACHED = -8
IS_MAX_TIME_REACHED = -9
IS_MAX_FUN_EVALS_REACHED = -10
IS_ALL_VARS_FIXED = -11
FAILED_TO_OBTAIN_MOVE_DIRECTION = -13
USER_DEMAND_EXIT = -99

FAILED_WITH_UNIMPLEMENTED_OR_UNKNOWN_REASON = -1000

def stopcase(arg):
    if hasattr(arg, 'istop'): istop = arg.istop
    else: istop = arg
    
    if istop > 0: return 1
    elif istop in [0, IS_MAX_ITER_REACHED, IS_MAX_CPU_TIME_REACHED, IS_MAX_TIME_REACHED, IS_MAX_FUN_EVALS_REACHED]: return 0
    else: return -1



def setDefaultIterFuncs(className):
    d = dict()

    # positive:
    d[SMALL_DF] = lambda p: small_df(p)
    d[SMALL_DELTA_X] = lambda p: small_deltaX(p)
    d[SMALL_DELTA_F] = lambda p: small_deltaF(p)
    d[FVAL_IS_ENOUGH] = lambda p: isEnough(p)
    #d[11] = lambda p: allVarsAreFixed(p)

    # negative:
    d[IS_NAN_IN_X]  = lambda p: isNanInX(p)
    d[IS_MAX_ITER_REACHED]  = lambda p: isMaxIterReached(p)
    d[IS_MAX_CPU_TIME_REACHED]  = lambda p: isMaxCPUTimeReached(p)
    d[IS_MAX_TIME_REACHED]  = lambda p: isMaxTimeReached(p)
    if className == 'NonLin': d[IS_MAX_FUN_EVALS_REACHED] = lambda p: isMaxFunEvalsReached(p)

    return d

def small_df(p):
    if not hasattr(p, '_df') or p._df is None: return False
    if hasattr(p._df, 'toarray'):p._df = p._df.toarray()
    if p.norm(p._df) >= p.gtol or not np.all(np.isfinite(p._df)) or not p.isFeas(p.iterValues.x[-1]): return False
    return (True, '|| gradient F(X[k]) || < gtol')
    #return False if not hasattr(p, 'dF') or p.dF == None or p.norm(p.dF) > p.gtol else True

def small_deltaX(p):
    if p.iter == 0: return False
    diffX = p.iterValues.x[-1] - p.iterValues.x[-2]
    if p.scale is not None: diffX *= p.scale
    if p.norm(diffX) >= p.xtol: return False
    else: return (True, '|| X[k] - X[k-1] || < xtol')
    #r = False if p.norm(p.xk - p.x_prev) > p.xtol else True


def small_deltaF(p):
    if p.iter == 0: return False
    #r = False if p.norm(p.fk - p.f_prev) > p.ftol else True
    if  np.isnan(p.iterValues.f[-1]) or np.isnan(p.iterValues.f[-2]) or\
    p.norm(p.iterValues.f[-1] - p.iterValues.f[-2]) >= p.ftol: # or (p.iterValues.r[-1] > p.contol and p.iterValues.r[-1] - p.iterValues.r[-2] < p.contol):
            return False
    else: return (True, '|| F[k] - F[k-1] || < ftol')


def isEnough(p):
#    asscalar(asarray(p.Fk)) was added for compatibility with ironpython
    if p.fEnough > np.asscalar(np.asarray(p.Fk)) and p.isFeas(p.xk): #TODO: mb replace by p.rk<p.contol? or other field like p.iterValues.isFeasible?
       return (True, 'fEnough has been reached')
    else: return False

def isNanInX(p):
    if np.any(np.isnan(p.xk)):
       return (True, 'NaN in X[k] coords has been obtained')
    else: return False


def isMaxIterReached(p):
    if p.iter >= p.maxIter-1: # iter numeration starts from zero
        return (True, 'Max Iter (p.maxIter=%d) has been reached' % p.maxIter)
    else: return False

def isMaxCPUTimeReached(p):
    if p.iterCPUTime[-1] < p.maxCPUTime + p.cpuTimeElapsedForPlotting[-1]:
        return False
    else:
        return (True, 'max CPU time limit has been reached')
    #return p.cpuTimeElapsed >= p.maxCPUTime

def isMaxTimeReached(p):
    if p.currtime - p.timeStart < p.maxTime + p.timeElapsedForPlotting[-1]:
        return False
    else:
        return (True, 'max time limit has been reached')

def isMaxFunEvalsReached(p):
    #if not hasattr(p, 'nFunEvals'): p.warn('no nFunEvals field'); return 0
    if p.nEvals['f'] >= p.maxFunEvals:
        return (True, 'max objfunc evals limit (p.maxFunEvals=%d) has been reached' %p.maxFunEvals)
    else:
        return False


####################################################################
def denyingStopFuncs(ProblemGroup=None):
    #mb in future it will be class-dependend,  like Matrix,  Nonlinear etc
    return {isMinIterReached : 'min iter is not reached yet', isMinTimeReached : 'min time is not reached yet', isMinCPUTimeReached:'min cputime is not reached yet', isMinFunEvalsReached:'min objective function evaluations nuber is not reached yet'}

isMinFunEvalsReached = lambda p: p.minFunEvals==0 or ('f' in p.nEvals.keys() and p.nEvals['f'] >= p.minFunEvals)

isMinTimeReached = lambda p: p.currtime - p.timeStart > p.minTime + p.timeElapsedForPlotting[-1]

isMinCPUTimeReached = lambda p: p.iterCPUTime[-1] >= p.minCPUTime + p.cpuTimeElapsedForPlotting[-1]

isMinIterReached = lambda p: p.iter >= p.minIter




from scipy.optimize import fmin_ncg
from openopt.kernel.ooMisc import isSolved
from openopt.kernel.baseSolver import baseSolver

class scipy_ncg(baseSolver):
    __name__ = 'scipy_ncg'
    __license__ = "BSD"
    #__authors__ =
    __alg__ = "Newton-CG"
    __info__ = 'unconstrained NLP solver, can handle 2nd derivatives'
    iterfcnConnected = True

    def __init__(self):pass
    def __solver__(self, p):

        def iterfcn(x):
            p.xk, p.fk = x, p.f(x)
            p.iterfcn()
            if p.istop: raise isSolved

        if p.userProvided.d2f: fhess = p.d2f
        else: fhess = None

        xf = fmin_ncg(p.f, p.x0, p.df, fhess = fhess, maxiter = p.maxIter+15, disp = 0, callback=iterfcn)

        ff = p.f(xf)
        p.istop = 1000
        p.xk = p.xf = xf
        p.fk = p.ff = ff




from numpy import isnan, take, any, all, logical_or, logical_and, logical_not, atleast_1d, \
asarray, argmin, argsort, abs, isfinite, dot#where
import numpy as np

# for PyPy
from openopt.kernel.nonOptMisc import where

from bisect import bisect_right
from FuncDesigner.Interval import splitDomainForDiscreteVariable

try:
    from bottleneck import nanmin
except ImportError:
    from numpy import nanmin

def getTruncatedArrays(ind, y, e, indT, _s):
    # TODO: rework it when numpy will have appropriate inplace function
    s = ind.size
    y = take(y, ind, axis=0, out=y[:s])
    e = take(e, ind, axis=0, out=e[:s])
    _s = _s[ind]
    if indT is not None:
        indT = indT[ind]
    return y, e, indT, _s#, nlh, nlh_0
    
def adjustDiscreteVarBounds(y, e, p):
    # TODO: rework it
    #n = p.n
    # TODO: remove the cycle, use vectorization
    for i in p._discreteVarsNumList:
        v = p._freeVarsList[i]
        y[:, i], e[:, i] = splitDomainForDiscreteVariable(y[:, i], e[:, i], v)

#    ind = y>e
#    assert not any(ind)
    
    
#    y[ind], e[ind] = e[ind], y[ind]

#    Ind = any(y>e, 1)
#    trunc_ind = where(logical_not(Ind))[0]

#    # TODO:  is it triggered? // updated: can be from MOP or cons
#    if any(Ind):
#        ind = where(logical_not(Ind))[0]
#        s = ind.size
#        y = take(y, ind, axis=0, out=y[:s])
#        e = take(e, ind, axis=0, out=e[:s])
#        _s = _s[ind]
#        if indT is not None:
#            indT = indT[ind]
    return y, e#, trunc_ind#_s, indT

    
def func7(y, e, o, a, _s, indT, nlhc, residual):
    r10 = logical_and(all(isnan(o), 1), all(isnan(a), 1))
    if any(r10):
        j = where(logical_not(r10))[0]
        lj = j.size
        y = take(y, j, axis=0, out=y[:lj])
        e = take(e, j, axis=0, out=e[:lj])
        o = take(o, j, axis=0, out=o[:lj])
        a = take(a, j, axis=0, out=a[:lj])
        _s = _s[j]
        if indT is not None:
            indT = indT[j]
        if nlhc is not None:
            nlhc = take(nlhc, j, axis=0, out=nlhc[:lj])
        if residual is not None:
            residual = take(residual, j, axis=0, out=residual[:lj])
    return y, e, o, a, _s, indT, nlhc, residual
    

def func9(an, fo, g, p):
    
    #ind = searchsorted(ar, fo, side='right')
    if p.probType in ('NLSP', 'SNLE') and p.maxSolutions != 1:
        mino = atleast_1d([node.key for node in an])
        ind = mino > 0
        if not any(ind):
            return an, g
        else:
            g = nanmin((g, nanmin(mino[ind])))
            ind2 = where(logical_not(ind))[0]
            #an = take(an, ind2, axis=0, out=an[:ind2.size])
            #an = asarray(an[ind2])
            an = [an[i] for i in ind2]
            return an, g
            
        
    elif p.solver.dataHandling == 'sorted':
        #OLD
        mino = [node.key for node in an]
        ind = bisect_right(mino, fo)
        if ind == len(mino):
            return an, g
        else:
            g = nanmin((g, nanmin(atleast_1d(mino[ind]))))
            return an[:ind], g
    elif p.solver.dataHandling == 'raw':
        
        #NEW
        mino = atleast_1d([node.key for node in an])
        r10 = mino > fo
        if not any(r10):
            return an, g
        else:
            ind = where(r10)[0]
            g = nanmin((g, nanmin(atleast_1d(mino)[ind])))
            #an = asarray(an)
            ind2 = where(logical_not(r10))[0]
            #an = take(an, ind2, axis=0, out=an[:ind2.size])
            an = [an[i] for i in ind2]
            return an, g

        # NEW 2
#        curr_tnlh = [node.tnlh_curr for node in an]
#        import warnings
#        warnings.warn('! fix g')
        
        return an, g
        
    else:
        assert 0, 'incorrect nodes remove approach'

def func5(an, nn, g, p):
    m = len(an)
    if m <= nn: return an, g
    
    mino = np.array([node.key for node in an])
    
    if nn == 1: # box-bound probs with exact interval analysis
        ind = argmin(mino)
        assert ind in (0, 1), 'error in interalg engine'
        g = nanmin((mino[1-ind], g))
        an = [an[i] for i in ind]
    elif m > nn:
        if p.solver.dataHandling == 'raw':
            ind = argsort(mino)
            th = mino[ind[nn]]
            ind2 = where(mino < th)[0]
            g = nanmin((th, g))
            #an = take(an, ind2, axis=0, out=an[:ind2.size])
            an = [an[i] for i in ind2]#an[ind2]
        else:
            g = nanmin((mino[nn], g))
            an = an[:nn]
    return an, g

def func4(p, y, e, o, a, fo, tnlhf_curr = None):
    # TODO: simplifications for all-bool probs
    if fo is None and tnlhf_curr is None: return False# used in IP
    if y.size == 0: return False
    cs = (y + e)/2
    n = y.shape[1]
    # TODO: a, o  could be chenged to +/- inf instead of values duplication
    
    if tnlhf_curr is not None:
        tnlh_modL = tnlhf_curr[:, :n]
        ind = logical_not(isfinite(tnlh_modL))
    else:
        s = o[:, :n]
        ind = logical_or(s > fo, isnan(s)) # TODO: assert isnan(s) is same to isnan(a_modL)
#    hasDiscreteVariables = len(p._discreteVarsNumList) != 0
    indT = any(ind, 1)
    if any(ind):
#        if hasDiscreteVariables:
        for j, v in enumerate(p._discreteVarsList):
            i = p._discreteVarsNumList[j]
            k = where(ind[:, i])[0]
            if k.size == 0: continue
            discr_mid1, discr_mid2 = splitDomainForDiscreteVariable(y[k, i], e[k, i], v)
            cs[k, i] = discr_mid2
        
        y[ind] = cs[ind]

        # TODO: check is it ever called from MOP, implement if not
        if p.probType != 'MOP': 
            a[:, :n][ind] = a[:, n:][ind]
            o[:, :n][ind] = o[:, n:][ind]
        if tnlhf_curr is not None:
            tnlhf_curr[:, :n][ind] = tnlhf_curr[:, n:][ind]


    if tnlhf_curr is not None:
        tnlh_modU = tnlhf_curr[:, n:]
        ind = logical_not(isfinite(tnlh_modU))
    else:
        q = o[:, n:]
        ind = logical_or(q > fo, isnan(q)) # TODO: assert isnan(q) is same to isnan(a_modU)
        
    indT = logical_or(any(ind, 1), indT)
    if any(ind):
        # copy is used to prevent y and e being same array, that may be buggy with discrete vars
        # TODO: remove copy after new discrete vars handling implementation
        for j, v in enumerate(p._discreteVarsList):
            i = p._discreteVarsNumList[j]
            k = where(ind[:, i])[0]
            if k.size == 0: continue
            discr_mid1, discr_mid2 = splitDomainForDiscreteVariable(y[k, i], e[k, i], v)
            cs[k, i] = discr_mid1
            
        e[ind] = cs[ind].copy() 
        # Changes
#        ind = logical_and(ind, logical_not(isnan(a[:, n:])))
##        ii = len(where(ind)[0])
##        if ii != 0: print ii
        if p.probType != 'MOP':
            a[:, n:][ind] = a[:, :n][ind]
            o[:, n:][ind] = o[:, :n][ind]
        if tnlhf_curr is not None:
            tnlhf_curr[:, n:][ind] = tnlhf_curr[:, :n][ind]
#        for arr in arrays:
#            if arr is not None:
#                arr[:, n:2*n][ind] = arr[:, 0:n][ind]
        
    return indT
    
def truncateByPlane(y, e, indT, A, b):
    #!!!!!!!!!!!!!!!!!!!
    # TODO: vectorize it by matrix A
    #!!!!!!!!!!!!!!!!!!!
    ind_trunc = True
#    assert np.asarray(b).size <= 1, 'unimplemented yet'
    m, n = y.shape
    if m == 0:
        assert e.size == 0, 'bug in interalg engine'
        return y, e, indT, ind_trunc

    ind_positive = where(A > 0)[0]
    ind_negative = where(A < 0)[0]
#    print ind_negative, ind_positive
    A1 = A[ind_positive] 
    S1 = A1 * y[:, ind_positive]
    A2 = A[ind_negative]
    S2 = A2 * e[:, ind_negative]
    
#    print A1.shape, A2.shape, e[:, ind_negative].shape
    S = S1.sum(axis=1) + S2.sum(axis=1)

    if ind_positive.size != 0:
        S1_ = b - S.reshape(-1, 1) + S1
        Alt_ub = S1_ / A1

        ind = e[:, ind_positive] > Alt_ub #+ 1e-15*abs(Alt_ub)
        if np.any(ind):
            _ind = where(ind.flatten())[0]
            e[:, ind_positive].flat[_ind] = Alt_ub.flat[_ind]
            indT[any(ind, axis = 1)] = True

    
    if ind_negative.size != 0:
        S2_ = b - S.reshape(-1, 1) + S2
        Alt_lb = S2_ / A2
        
        ind = y[:, ind_negative] < Alt_lb #- 1e-15 * abs(Alt_lb)
        if np.any(ind):
            #y[:, ind_negative][ind] = Alt_lb[ind]
            _ind = where(ind.flatten())[0]
            y[:, ind_negative].flat[_ind] = Alt_lb.flat[_ind]
            indT[any(ind, axis = 1)] = True


    ind = all(e>=y, axis = 1)
    if not all(ind):
        ind_trunc = where(ind)[0]
        lj = ind_trunc.size
        y = take(y, ind_trunc, axis=0, out=y[:lj])
        e = take(e, ind_trunc, axis=0, out=e[:lj])
        indT = indT[ind_trunc]
            
    return y, e, indT, ind_trunc


def _truncateByPlane(y, e, indT, A, b):
    #!!!!!!!!!!!!!!!!!!!
    # TODO: vectorize it by matrix A
    #!!!!!!!!!!!!!!!!!!!
    ind_trunc = slice(None)
#    assert np.asarray(b).size <= 1, 'unimplemented yet'
    m, n = y.shape
    A = A.T
    assert A.shape == y.shape
    if m == 0:
        assert e.size == 0, 'bug in interalg engine'
        return y, e, indT, ind_trunc

#    ind_positive = where(A > 0)[0]
#    ind_negative = where(A < 0)[0]
#    print ind_negative, ind_positive

    #TODO: mb store ind_positive, ind_negative in prob for fixed A

    ind_positive, ind_negative = where(A>0), where(A<0)
#    S1 = dot(where(A>0, A, -A).T, y.T)
#    S2 = dot(where(A<0, A, -A).T, e.T)
    
#    S_y = dot(where(A>0, A, 0).T, y.T)
#    S_y_ = dot(where(A<0, A, 0).T, y.T)
#    S_e = dot(where(A>0, A, 0).T, e.T)
#    S_e_ = dot(where(A<0, A, 0).T, e.T)

    S_y = where(A>0, A, 0) * y
    S_y_ = where(A<0, A, 0) * y
    S_e = where(A>0, A, 0) * e
    S_e_ = where(A<0, A, 0) * e
    

#    _S = S1+S2

    # 1
    S1 = S_y 
    S2 = S_e_
    S = S1.sum(axis=1) + S2.sum(axis=1)
    d = (b.reshape(-1, 1)  - S.reshape(-1, 1)) + S1 # vector + matrix
    Alt = (d / A) # vector / matrix
    #Alt = 
    
    ind = e[ind_positive] > Alt[ind_positive] + 1e-15*(abs(Alt[ind_positive])+abs(e[ind_positive]))
    
    if np.any(ind):
        _ind = where(ind.flatten())[0]
        e0 = e.copy()# TODO: mb rework it
        e[ind_positive].flat[_ind] = Alt.flat[_ind]
        ind_ = any(e!=e0, axis = 1)
#        indT[any(ind_, axis = 1)] = True
        indT[ind_] = True
    
    # 2
    S1 = S_y_
    S2 = S_e
    S = S1.sum(axis=1) + S2.sum(axis=1)
    
    d = (b.reshape(-1, 1)  - S.reshape(-1, 1)) + S2 # vector + matrix
    Alt = (d / A) # vector / matrix
    
    ind = y[ind_negative] < Alt[ind_negative] - 1e-15 * (abs(Alt[ind_negative]) + abs(y[ind_negative]))
    if np.any(ind):
        #y[:, ind_negative][ind] = Alt[ind]
        _ind = where(ind.flatten())[0]
        y0 = y.copy()# TODO: mb rework it
        y[ind_negative].flat[_ind] = Alt.flat[_ind]
        ind_ = any(y!=y0, axis = 1)
#        indT[any(ind_, axis = 1)] = True
        indT[ind_] = True
    # 3

    ind = all(e>=y, axis = 1)
    if not all(ind):
        ind_trunc = where(ind)[0]
        lj = ind_trunc.size
        y = take(y, ind_trunc, axis=0, out=y[:lj])
        e = take(e, ind_trunc, axis=0, out=e[:lj])
        indT = indT[ind_trunc]
    
    return y, e, indT, ind_trunc
    
def truncateByPlane2(cs, centerValues, y, e, indT, gradient, fo, p):
    
    ind_trunc = True
    assert np.asarray(fo).size <= 1, 'unimplemented yet'
    m, n = y.shape
    if m == 0:
        assert e.size == 0, 'bug in interalg engine'
        return y, e, indT, ind_trunc

    oovarsIndDict = p._oovarsIndDict
    ind = np.array([oovarsIndDict[oov][0] for oov in gradient.keys()])
    y2, e2 = y[:, ind], e[:, ind]
    
    A = np.vstack([np.asarray(elem).reshape(1, -1) for elem in gradient.values()]).T
    cs = 0.5 * (y2 + e2)
    b = np.sum(A * cs, 1) - centerValues.view(np.ndarray) + fo

#    ind_positive = where(A > 0)
#    ind_negative = where(A < 0)
    
    A_positive = where(A>0, A, 0)
    A_negative = where(A<0, A, 0)
    #S1 = A[ind_positive] * y2[ind_positive]
    #S2 = A[ind_negative] * e2[ind_negative]
    S1 = A_positive * y2
    S2 = A_negative * e2
    s1, s2 = np.sum(S1, 1), np.sum(S2, 1)
    S = s1 + s2
    
    alt_fo1 = where(A_positive != 0, (b.reshape(-1, 1) - S.reshape(-1, 1) + S1) / A_positive, np.inf)
    ind1 = logical_and(e2 > alt_fo1, A_positive != 0)
    e2[ind1] = alt_fo1[ind1]
    
    alt_fo2 = where(A_negative != 0, (b.reshape(-1, 1) - S.reshape(-1, 1) + S2) / A_negative, -np.inf)
    ind2 = logical_and(y2 < alt_fo2, A_negative != 0)
    y2[ind2] = alt_fo2[ind2]
    
    
    # TODO: check it
    y[:, ind], e[:, ind] = y2, e2
    
    # TODO: check indT
    indT[np.any(ind1, 1)] = True
    indT[np.any(ind2, 1)] = True
    
#    for _i, i in enumerate(ind_positive):
#        s = S - S1[:, _i]
#        alt_ub = (b - s) / A[i]
#        ind = e[:, i] > alt_ub
#        e[ind, i] = alt_ub[ind]
#        indT[ind] = True
#    
#    for _i, i in enumerate(ind_negative):
#        s = S - S2[:, _i]
#        alt_lb = (b - s) / A[i]
#        ind = y[:, i] < alt_lb
#        y[ind, i] = alt_lb[ind]
#        indT[ind] = True

    ind = np.all(e>=y, 1)
    if not np.all(ind):
        ind_trunc = where(ind)[0]
        lj = ind_trunc.size
        y = take(y, ind_trunc, axis=0, out=y[:lj])
        e = take(e, ind_trunc, axis=0, out=e[:lj])
        indT = indT[ind_trunc]
    
    return y, e, indT, ind_trunc
    
    
def  truncateByConvexFunc():
    if 0 and (p._linear_objective or p.convex in (1, True)) and fo_prev < 1e300:# and p.convex is True:
    # TODO: rework it
    #cs = dict([(key, val.view(multiarray)) for key, val in cs.items()])
    #cs = dict([(key, 0) for key, val in p._x0.items()])
    

        # TODO: handle indT corrently
        indT2 = np.empty(y.shape[0])
        indT2.fill(False)

        #y, e, indT2 = truncateByPlane(y, e, indT2, np.hstack([d[v][0] for v in vv]), fo_prev)
        #y, e, indT2, ind_t = truncateByPlane(y, e, indT2, np.hstack([d[oov] for oov in vv]), fo_prev)
        
    #        print('==')
    #        print(y.sum())
        if p._linear_objective:
            d = p._linear_objective_factor
            th = fo_prev + (p._linear_objective_scalar if p.goal not in ('min', 'minimum') else - p._linear_objective_scalar)
            y, e, indT2, ind_t = truncateByPlane(y, e, indT2, d if p.goal in ('min', 'minimum') else -d, th)
        elif p.convex in (1, True):

            assert p.goal in ('min', 'minimum') 
            wr4 = (y+e) / 2
            adjustr4WithDiscreteVariables(wr4, p)
            #cs = dict([(oovar, asarray((y[:, i]+e[:, i])/2, dataType)) for i, oovar in enumerate(vv)])
            cs = dict([(oovar, asarray(wr4[:, i], dataType).view(multiarray)) for i, oovar in enumerate(vv)])            
            #cs = dict([(key, val.view(multiarray)) for key, val in cs.items()])
            
            #TODO: add other args
            centerValues = asdf1(cs)
            gradient = asdf1.D(cs)
            y, e, indT2, ind_t = truncateByPlane2(cs, centerValues, y, e, indT2, gradient, fo_prev, p)
        else:
            assert 0, 'bug in FD kernel'
            
        if ind_t is not True:
            lj = ind_t.size
            o = take(o, ind_t, axis=0, out=o[:lj])
            a = take(a, ind_t, axis=0, out=a[:lj])
            if nlhc is not None:
                nlhc = take(nlhc, ind_t, axis=0, out=nlhc[:lj])
                indTC = np.logical_or(indTC[ind_t], indT2)
            _s = _s[ind_t]
            #residual = residual[ind_t]
        









#!/usr/bin/python
'''
A simple OpenOpt multiobjective TSP example for directed multigraph using interalg solver;
requires networkx (http://networkx.lanl.gov)
and FuncDesigner installed.
For some solvers limitations on time, cputime, "enough" value, basic GUI features are available.
See http://openopt.org/TSP for more details
'''
from openopt import *
from numpy import sin, cos#, abs

import networkx as nx
N = 6
G = nx.MultiDiGraph()

G.add_edges_from(\
                 [(i,j,{'time': 1.5*(cos(i)+sin(j)+1)**2, 'cost':(i-j)**2 + 2*sin(i) + 2*cos(j)+1, 'way': 'aircraft'}) for i in range(N) for j in range(N) if i != j ])

G.add_edges_from(\
                 [(i,j,{'time': 4.5*(cos(i)-sin(j)+1)**2, 'cost':(i-j)**2 + sin(i) + cos(j)+1, 'way': 'railroad'}) for i in range(int(2*N/3)) for j in range(int(N)) if i != j ])

G.add_edges_from(\
                 [(i,j,{'time': 4.5*(cos(4*i)-sin(3*j)+1)**2, 'cost':(i-2*j)**2 + sin(10+i) + cos(2*j)+1, 'way': 'car'}) for i in range(int(2*N/3)) for j in range(int(N)) if i != j ])

G.add_edges_from(\
                 [(i,j,{'time': +(4.5*(cos(i)+cos(j)+1)**2 + abs(i - j)), 'cost': (0.2*i + 0.1*j)**2, 'way': 'bike'}) for i in range(int(N)) for j in range(int(N)) if i != j ])

objective = [
              # name, tol, goal
              'time', 0.005, 'min', 
              'cost', 0.005, 'min'
              ]

# for solver sa handling of constraints is unimplemented yet
# when your solver is interalg you can use nonlinear constraints as well
# for nonlinear objective and constraints functions like arctan, abs etc should be imported from FuncDesigner
from FuncDesigner import arctan, sqrt
constraints = lambda value: (
                             2 * value['time']**2 + 3 * sqrt(value['cost']) < 10000, 
                             8 * arctan(value['time']) + 15*value['cost'] > 15
                             )

p = TSP(G, objective = objective, constraints = constraints)
r = p.solve('interalg', nProc=2) # see http://openopt.org/interalg for more info on the solver

# you can provide some stop criterion, 
# e.g. maxTime, maxCPUTime, fEnough etc, for example 
# r = p.solve('interalg', maxTime = 100, maxCPUTime=100) 

# also you can use p.manage() to enable basic GUI (http://openopt.org/OOFrameworkDoc#Solving) 
# it requires tkinter installed, that is included into PythonXY, EPD;
# for linux use [sudo] easy_install tk or [sodo] apt-get install python-tk
#r = p.manage('interalg')

print(r.solutions.values)
print(r.solutions)
'''
istop: 1001 (all solutions have been obtained)
Solver:   Time Elapsed = 319.53 	CPU Time Elapsed = 297.35
4 solutions have been obtained
[[ 41.86372151   8.53983205]
 [ 56.35337592   4.51      ]
 [ 36.67772554  22.73951982]
 [ 34.649273    31.5852164 ]]
...
'''



#! /usr/bin/env python

descr   = """
"""

import os
import sys

DISTNAME            = 'openopt'
DESCRIPTION         = 'A python module for numerical optimization'
LONG_DESCRIPTION    = descr
MAINTAINER          = 'Dmitrey Kroshko'
MAINTAINER_EMAIL    = 'dmitrey-at-openopt-dot-org'
URL                 = 'http://openopt.org'
LICENSE             = 'BSD'

sys.path.append(os.getcwd() + os.sep + 'openopt')
from ooVersionNumber import __version__ as ooVer
openopt_version = ooVer

#DOWNLOAD_URL        = 'http://openopt.org/images/3/33/OpenOpt.zip'

try:
    import setuptools
except:
    print('you should have setuptools installed (http://pypi.python.org/pypi/setuptools), for some Linux distribs you can get it via [sudo] apt-get install python-setuptools')
    print('press Enter for exit...')
    raw_input()
    exit()
    
#from numpy.distutils.system_info import system_info, NotFoundError, dict_append, so_ext
from numpy.distutils.core import setup

DOC_FILES = []

def configuration(parent_package='',top_path=None, package_name=DISTNAME):
    if os.path.exists('MANIFEST'): os.remove('MANIFEST')
    #pkg_prefix_dir = 'openopt'

    # Get the version

    from numpy.distutils.misc_util import Configuration
    config = Configuration(package_name,parent_package,top_path,
        version     = openopt_version,
        maintainer  = MAINTAINER,
        maintainer_email = MAINTAINER_EMAIL,
        description = DESCRIPTION,
        license = LICENSE,
        url = URL,
        #download_url = DOWNLOAD_URL,
        long_description = LONG_DESCRIPTION)


    # XXX: once in SVN, should add svn version...
    #print config.make_svn_version_py()

    # package_data does not work with sdist for setuptools 0.5 (setuptools bug),
    # so we need to add them here while the bug is not solved...

    return config


if __name__ == "__main__":

    # setuptools version of config script

    # package_data does not work with sdist for setuptools 0.5 (setuptools bug)
    # So we cannot add data files via setuptools yet.

    #data_files = ['test_data/' + i for i in TEST_DATA_FILES]
    #data_files.extend(['docs/' + i for i in doc_files])
    setup(configuration = configuration,
        install_requires=['numpy','setproctitle'], # can also add version specifiers   #namespace_packages=['kernel'],
        #setup_requires = 'setproctitle', 
        #py_modules = ['kernel', 'tests', 'examples', 'solvers'],
        packages=setuptools.find_packages(),
        include_package_data = True,
        #package_data = '*.txt',
        test_suite='',#"openopt.tests", # for python setup.py test
        zip_safe=False, # the package can run out of an .egg file
        #FIXME url, download_url, ext_modules
        classifiers =
            [ 'Development Status :: 5 - Production/Stable',
              'Environment :: Console',
              'Intended Audience :: Developers',
              'Intended Audience :: Science/Research',
              'License :: OSI Approved :: BSD License',
              'Topic :: Scientific/Engineering']
    )



__docformat__ = "restructuredtext en"

from time import time, clock
from numpy import isscalar,  array_equal

######################
# don't change to mere ooMisc! 
from openopt.kernel.ooMisc import isSolved 
######################

from setDefaultIterFuncs import USER_DEMAND_STOP, IS_NAN_IN_X, SMALL_DELTA_X, IS_MAX_ITER_REACHED, \
IS_MAX_CPU_TIME_REACHED, IS_MAX_TIME_REACHED, IS_MAX_FUN_EVALS_REACHED

#has_Tkinter = True
#try:
#    import Tkinter
#except ImportError:
#    has_Tkinter = False

NoneType = type(None)

def ooIter(p, *args,  **kwargs):
    """
    this func is called from iter to iter
    it is default iter function of OpenOpt kernel
    lots of solvers use this one
    it provides basic graphics output (provided plot option is turned on),
    maybe in future some text output will also be generated here.
    also, some stop criteria are handled via the func.
    """
    
    if p.finalIterFcnFinished: return

    if p.state == 'paused':
        p.GUI_root.wait_variable(p.statusTextVariable)

    if not hasattr(p, 'timeStart'): return#called from check 1st derivatives
    
    p.currtime = time()
    if not p.iter:
        p.lastDrawTime = p.currtime
        p.lastDrawIter = 0

    if not p.isFinished or len(p.iterValues.f) == 0:
        p.solver.__decodeIterFcnArgs__(p,  *args,  **kwargs)
        condEqualLastPoints = hasattr(p, 'xk_prev') and array_equal(p.xk,  p.xk_prev) 
        p.xk_prev = p.xk.copy()
        if p.graphics.xlabel == 'nf': p.iterValues.nf.append(p.nEvals['f'])
        p.iterCPUTime.append(clock() - p.cpuTimeStart)
        p.iterTime.append(p.currtime - p.timeStart)
        
        if p.isManagerUsed:
            if p.iterTime[-1] - p._manager_time > 1:
                p._manager_time_variable.set('%d' % int(p.iterTime[-1]))
                p._manager_time = p.iterTime[-1]
                if p.iterCPUTime[-1] - p._manager_cputime > 1:
                    p._manager_cputime_variable.set('%d' % int(p.iterCPUTime[-1]))
                    p._manager_cputime = p.iterCPUTime[-1]

        # TODO: rework it
        if p.probType not in ('GLP', 'MILP') and p.solver.__name__ not in ('de', 'pswarm', 'interalg') \
        and ((p.iter == 1 and array_equal(p.xk,  p.x0)) or condEqualLastPoints):
            elems = [getattr(p.iterValues,  fn) for fn in dir(p.iterValues)] + [p.iterTime, p.iterCPUTime]#dir(p.iterValues)
            for elem in elems:
                if type(elem) == list:
                    elem.pop(-1)

            #TODO: handle case x0 = x1 = x2 = ...
            if not (p.isFinished and condEqualLastPoints): 
                return

        #TODO: turn off xtol and ftol for artifically iterfcn funcs

        if not p.userStop and (not condEqualLastPoints or p.probType == 'GLP' or p.solver.__name__ in ('de', 'pswarm', 'interalg')):
            for key, fun in p.kernelIterFuncs.items():
                r =  fun(p)
                if r is not False:
                    p.stopdict[key] = True
                    if p.istop == 0 or key not in [IS_MAX_ITER_REACHED, IS_MAX_CPU_TIME_REACHED, IS_MAX_TIME_REACHED, IS_MAX_FUN_EVALS_REACHED]:
                        p.istop = key
                        if type(r) == tuple:
                            p.msg = r[1]
                        else:
                            p.msg = 'unkown, if you see the message inform openopt developers'
            if IS_NAN_IN_X in p.stopdict:pass
            elif SMALL_DELTA_X in p.stopdict and array_equal(p.iterValues.x[-1], p.iterValues.x[-2]): pass
            else:
                p.nonStopMsg = ''
                for fun in p.denyingStopFuncs.keys():
                    if not fun(p):
                        p.istop = 0
                        p.stopdict = {}
                        p.msg = ''
                        p.nonStopMsg = p.denyingStopFuncs[fun]
                        break
                for fun in p.callback:
                    r =  fun(p)
                    if r is None: p.err('user-defined callback function returned None, that is forbidden, see /doc/userCallback.py for allowed return values')
                    if r not in [0,  False]:
                        if r in [True,  1]:  p.istop = USER_DEMAND_STOP
                        elif isscalar(r):
                            p.istop = r
                            p.msg = 'user-defined'
                        else:
                            p.istop = r[0]
                            p.msg = r[1]
                        p.stopdict[p.istop] = True
                        p.userStop = True
    if not p.solver.properTextOutput: 
        p.iterPrint()
    T, cpuT = 0., 0.

    if p.plot and (p.iter == 0 or p.iter <2 or p.isFinished or \
    p.currtime - p.lastDrawTime > p.graphics.rate * (p.currtime - p.iterTime[p.lastDrawIter] - p.timeStart)):
        for df in p.graphics.drawFuncs: df(p)
        T = time() - p.timeStart - p.iterTime[-1]
        cpuT = clock() - p.cpuTimeStart - p.iterCPUTime[-1]
        p.lastDrawTime = time()
        p.lastDrawIter = p.iter
    if p.plot:
        p.timeElapsedForPlotting.append(T+p.timeElapsedForPlotting[-1])
        p.cpuTimeElapsedForPlotting.append(cpuT+p.cpuTimeElapsedForPlotting[-1])
    
    p.iter += 1

    if p.isFinished: p.finalIterFcnFinished = True
    if p.istop and p.istop != 1000 and not p.solver.iterfcnConnected and not p.isFinished and p.solver.useStopByException:
        p.debugmsg('exit solver via exception; istop=%d' % p.istop)
        raise isSolved





PythonMin, PythonMax = min, max
from numpy import isnan, array, atleast_1d, all, searchsorted, logical_or, any, nan, \
vstack, inf, where, logical_not, min, abs, insert, logical_xor, argsort, zeros_like

# for PyPy
from openopt.kernel.nonOptMisc import isPyPy

try:
    from numpy import append
except ImportError:
    def append(*args, **kw):
        raise ImportError('function append() is absent in PyPy yet')
        
from interalgLLR import *

try:
    from bottleneck import nanmin, nanmax
except ImportError:
    from numpy import nanmin, nanmax

hasPoint = lambda y, e, point:\
    True if y.size != 0 and any([(all(y[i]<=point) and all(e[i]>=point)) for i in range(y.shape[0])]) else False
pointInd = lambda y, e, point:\
    where([(all(y[i]<=point) and all(e[i]>=point)) for i in range(y.shape[0])])[0].tolist()
    
def r14(p, nlhc, residual, definiteRange, y, e, vv, asdf1, C, 
                 r40, g, nNodes, 
                 r41, fTol, Solutions, varTols, _in, dataType,
                 maxNodes, _s, indTC, xRecord):

    isSNLE = p.probType in ('NLSP', 'SNLE')
    
#    print p.iter, y, e, hasPoint(y, e, [10.,  20.,  10.,  50])

    #maxSolutions, solutions, coords = Solutions.maxNum, Solutions.solutions, Solutions.coords
    maxSolutions, solutions = Solutions.maxNum, Solutions.solutions

    fo_prev = \
    float(0 if isSNLE else PythonMin((r41, r40 - (fTol if maxSolutions == 1 else 0))))
    fo_prev  =  PythonMin(1e300, fo_prev)

    if 0 and isSNLE:
        a = residual
        o = zeros_like(a)
        o2, a2, r41 = r45(y, e, vv, p, asdf1, dataType, r41, nlhc)
#        print p.iter,nanmax(a/a2) / nanmin(a/a2)
#        o, a = o2, a2
        a = a2
        o = zeros_like(a)
    else:
        y, e, o, a, r41, _s, indTC = \
        r45(y, e, vv, p, asdf1, dataType, r41, nlhc, _s, indTC)
        #assert _s.size == y.shape[0]
    
    if o is None:
        return _in, g, fo_prev, _s, Solutions, xRecord, r41, r40
    
#    print(o, a)
#    print 'LF:', o
#    print 'a:', a
#    if p.iter == 13:
#        print y[1]
#        print e[1]
#        print o[1]
#        print a[1]

#    if not isSNLE and isfinite(r40):
#        tmp1 = r40
#        tmp2 = PythonMin(nanmin(o), PythonMin([1e300]+[node.key for node in _in]))
#        rTol = p.rTol * (tmp1 - tmp2) / (1e-300+PythonMin(abs(tmp1), abs(tmp2)))
#        fTol = PythonMax(rTol, fTol)
#        print p.iter, fTol
    
    y, e, o, a, _s, indTC, nlhc, residual = \
    func7(y, e, o, a, _s, indTC, nlhc, residual)    

    if y.size == 0:
        return _in, g, fo_prev, _s, Solutions, xRecord, r41, r40
    
    nodes = func11(y, e, nlhc, indTC, residual, o, a, _s, p)
    #nodes, g = func9(nodes, fo_prev, g, p)
    #y, e = func4(y, e, o, a, fo)
    

    if p.solver.dataHandling == 'raw':
            
        if not isSNLE:
            tmp = o.copy()
            tmp[tmp > fo_prev] = -inf
            M = atleast_1d(nanmax(tmp, 1))
            for i, node in enumerate(nodes):
                node.th_key = M[i]
                node.fo = fo_prev       
                
        if 0 or p.__isNoMoreThanBoxBounded__():#nlhc is not None and not isSNLE:
            for node in nodes: node.tnlhf = node.nlhf 
        else:
            for node in nodes: node.tnlhf = node.nlhc + node.nlhf 
            
        an = nodes + _in#hstack((nodes, _in))
        
        #tnlh_fixed = vstack([node.tnlhf for node in an])
        tnlh_fixed_local = vstack([node.nlhf for node in nodes])#tnlh_fixed[:len(nodes)]
#        if isSNLE:
#            tnlh_curr = tnlh_fixed_local#vstack([node.nlhc for node in nodes])
        if 1:
            tmp_u = where(a>fo_prev, fo_prev, a)#a.copy()
            tmp_l = where(o==-inf, -1e300, o)
#            tmp[tmp>fo_prev] = fo_prev
            tmp2 = tmp_u - tmp_l
#            ind_inf = tmp2==inf
            tmp2[tmp2<1e-300] = 1e-300
            o_exclude_ind = o > fo_prev
            if any(o_exclude_ind):
                tmp2[o_exclude_ind] = nan
                if not isSNLE:
                    g = PythonMin(g, min(o[o_exclude_ind]))
            tnlh_curr = tnlh_fixed_local - log2(tmp2)
#            tnlh_curr[ind_inf] = 1e300
        else:
            if isSNLE:
                tnlh_curr = tnlh_fixed_local
            else:
                tmp = a.copy()
                tmp[tmp>fo_prev] = fo_prev
                tmp2 = tmp - o
                tmp2[tmp2<1e-300] = 1e-300
                tmp2[o > fo_prev] = nan
                tnlh_curr = tnlh_fixed_local - log2(tmp2)
        tnlh_curr_best = nanmin(tnlh_curr, 1)
        for i, node in enumerate(nodes):
            node.tnlh_curr = tnlh_curr[i]
            node.tnlh_curr_best = tnlh_curr_best[i]
        
        # TODO: use it instead of code above
        #tnlh_curr = tnlh_fixed_local - log2(where() - o)
    else:
        tnlh_curr = None
    
    # TODO: don't calculate PointVals for zero-p regions
    PointVals, PointCoords = getr4Values(vv, y, e, tnlh_curr, asdf1, C, p.contol, dataType, p) 
    if PointVals.size != 0:
        xk, Min = r2(PointVals, PointCoords, dataType)
    else: # all points have been removed by func7
        xk = p.xk
        Min = nan

    if r40 > Min:
        r40 = Min
        xRecord = xk.copy()# TODO: is copy required?
    
    r41 = PythonMin(Min, r41)
    
    fo = \
    float(0 if isSNLE else PythonMin(r41, r40 - (fTol if maxSolutions == 1 else 0)))

    tmp = array([node.key for node in (nodes + _in)])
    f_bound_estimation = tmp.min()
    if isfinite(r40):
        p.f_bound_distance = r40 - f_bound_estimation
    if p.goal in ('max', 'maximum'):
        f_bound_estimation = -f_bound_estimation
    p.f_bound_estimation = f_bound_estimation
    
#    print '!', p.iter, y, e, hasPoint(y, e, [10.,  20.,  10.,  50])

    if p.solver.dataHandling == 'raw':
        
        if fo != fo_prev and not  isSNLE:
            fos = array([node.fo for node in an])
            
            #prev
            #ind_update = where(fos > fo + 0.01* fTol)[0]
            
            #new
            th_keys = array([node.th_key for node in an])
            delta_fos = fos - fo
            ind_update = where(10 * delta_fos > fos - th_keys)[0]
            
            update_nlh = True if ind_update.size != 0 else False
#                  print 'o MB:', float(o_tmp.nbytes) / 1e6
#                  print 'percent:', 100*float(ind_update.size) / len(an) 
            if update_nlh:
                #nodesToUpdate = an[ind_update]
                nodesToUpdate = [an[i] for i in ind_update]
#                    from time import time
#                    tt = time()
                updateNodes(nodesToUpdate, fo)
#                    if not hasattr(p, 'Time'):
#                        p.Time = time() - tt
#                    else:
#                        p.Time += time() - tt
                    
            tmp = array([node.key for node in an])
            cond = tmp > fo
            r10 = where(cond)[0]
            g = PythonMin([an[i].key for i in r10]+[g])
            ind_remain = where(logical_not(cond))[0]
            an = [an[i] for i in ind_remain]

        NN = atleast_1d([node.tnlh_curr_best for node in an])
        r10 = logical_or(isnan(NN), NN == inf)

        if any(r10):
            ind = where(logical_not(r10))[0]
            an = [an[i] for i in ind]#an[ind]
            #tnlh = take(tnlh, ind, axis=0, out=tnlh[:ind.size])
            #NN = take(NN, ind, axis=0, out=NN[:ind.size])
            NN = NN[ind]

        if 1 or not isSNLE or p.maxSolutions == 1:
            #pass
            astnlh = argsort(NN)
#            print(astnlh[:10])
            an = [an[i] for i in astnlh]#an[astnlh]
            
#        print(an[0].nlhc, an[0].tnlh_curr_best)
        # Changes
#        if NN.size != 0:
#            ind = searchsorted(NN, an[0].tnlh_curr_best+1)
#            tmp1, tmp2 = an[:ind], an[ind:]
#            arr = [node.key for node in tmp1]
#            Ind = argsort(arr)
#            an = hstack((tmp1[Ind], tmp2))
        #print [node.tnlh_curr_best for node in an[:10]]
    
    else: #if p.solver.dataHandling == 'sorted':
        if isSNLE and p.maxSolutions != 1: 
            an = nodes + _in#hstack((nodes, _in))
        elif isPyPy:
            an = nodes + _in
            an.sort(key = lambda obj: obj.key)
        else:
            nodes.sort(key = lambda obj: obj.key)

            if len(_in) == 0:
                an = nodes
            else:
                arr1 = [node.key for node in _in]
                arr2 = [node.key for node in nodes]
                r10 = searchsorted(arr1, arr2)
                an = insert(_in, r10, nodes).tolist()
#                if p.debug:
#                    arr = array([node.key for node in an])
#                    #print arr[0]
#                    assert all(arr[1:]>= arr[:-1])

    if maxSolutions != 1:
        Solutions = r46(PointCoords, PointVals, fTol, varTols, Solutions)
        
        p._nObtainedSolutions = len(solutions)
        if p._nObtainedSolutions > maxSolutions:
            solutions = solutions[:maxSolutions]
            p.istop = 0
            p.msg = 'user-defined maximal number of solutions (p.maxSolutions = %d) has been exeeded' % p.maxSolutions
            return an, g, fo, None, Solutions, xRecord, r41, r40

    p.iterfcn(xRecord, r40)
    if not isSNLE and isfinite(r41) and len(an):
        tmp = array([node.key for node in an])
        cond_exclude = r40 - tmp <= \
        p.rTol * where(abs(tmp) < abs(r40), abs(tmp), abs(r40))
        g = PythonMin([g] + [an[j].key for j in where(cond_exclude)[0]])
        cond_remain = logical_not(cond_exclude)
        ind = where(cond_remain)[0]
        an = [an[i] for i in ind]
    
    if p.istop != 0: 
        return an, g, fo, None, Solutions, xRecord, r41, r40
    if isSNLE and maxSolutions == 1 and Min <= fTol:
        # TODO: rework it for nonlinear systems with non-bound constraints
        p.istop, p.msg = 1000, 'required solution has been obtained'
        return an, g, fo, None, Solutions, xRecord, r41, r40
    
    an, g = func9(an, fo, g, p)
        
    nn = maxNodes#1 if asdf1.isUncycled and all(isfinite(o)) and p._isOnlyBoxBounded and not p.probType.startswith('MI') else maxNodes

    an, g = func5(an, nn, g, p)
    nNodes.append(len(an))
        
    return an, g, fo, _s, Solutions, xRecord, r41, r40


def r46(PointCoords, PointVals, fTol, varTols, Solutions):
    solutions, coords = Solutions.solutions, Solutions.coords
    
    r5Ind =  where(PointVals < fTol)[0]

    r5 = PointCoords[r5Ind]
    
    for c in r5:
        if len(solutions) == 0 or not any(all(abs(c - coords) < varTols, 1)): 
            solutions.append(c)
            #coords = asarray(solutions)
            Solutions.coords = append(Solutions.coords, c.reshape(1, -1), 0)
            
    return Solutions


def r45(y, e, vv, p, asdf1, dataType, r41, nlhc, _s, indT):
    y, e, o, a, definiteRange, exactRange, _s, indT = func82(y, e, vv, asdf1, dataType, p, 
                                                      r41, _s, indT)
    if o is None:
        return y, e, o, a, r41, _s, indT
    if p.debug and any(a + 1e-15 < o):  
        p.warn('interval lower bound exceeds upper bound, it seems to be FuncDesigner kernel bug')
    if p.debug and any(logical_xor(isnan(o), isnan(a))):
        p.err('bug in FuncDesigner intervals engine')
    
    n = p.n
    m = o.size // (2*n)
    o, a = o.reshape(2*n, m).T, a.reshape(2*n, m).T
    assert len(_s) == m
    
    if p.probType not in ('SNLE', 'NLSP') and asdf1.isUncycled and not p.probType.startswith('MI') \
    and len(p._discreteVarsList)==0 and exactRange:# for SNLE fo = 0
        # TODO: 
        # handle constraints with restricted domain and matrix definiteRange
        ind = where(definiteRange)[0]
        
        if ind.size != 0:
            o2 = o[ind]
            if nlhc is not None:
                nlhc = nlhc[ind]
            # TODO: if o has at least one -inf => prob is unbounded
            tmp1 = o2[nlhc==0] if nlhc is not None else o2
            if tmp1.size != 0:
                tmp1 = nanmin(tmp1)
                
                ## to prevent roundoff issues ##
                tmp1 += 1e-14*abs(tmp1)
                if tmp1 == 0: tmp1 = 1e-300 
                ######################
                
                r41 = nanmin((r41, tmp1)) 
        
    return y, e, o, a, r41, _s, indT

def updateNodes(nodesToUpdate, fo):
    if len(nodesToUpdate) == 0: return
    a_tmp = vstack([node.a for node in nodesToUpdate])
    Tmp = a_tmp
    Tmp[Tmp>fo] = fo                

    o_tmp = vstack([node.o for node in nodesToUpdate])
    Tmp -= o_tmp
    Tmp[Tmp<1e-300] = 1e-300
    Tmp[o_tmp>fo] = nan
    tnlh_all_new =  - log2(Tmp)
    
    del Tmp, a_tmp
    
    tnlh_all_new += vstack([node.tnlhf for node in nodesToUpdate])#tnlh_fixed[ind_update]
    
    tnlh_curr_best = nanmin(tnlh_all_new, 1)

    o_tmp[o_tmp > fo] = -inf
    M = atleast_1d(nanmax(o_tmp, 1))
    for j, node in enumerate(nodesToUpdate): 
        node.fo = fo
        node.tnlh_curr = tnlh_all_new[j]
        node.tnlh_curr_best = tnlh_curr_best[j]
        node.th_key = M[j]

#    return tnlh_all_new, tnlh_curr_best, M


#from multiprocessing import Pool
#from numpy import array_split
#def updateNodes(nodesToUpdate, fo, p):
#    if p.nProc == 1:
#        Chunks = [nodesToUpdate]
#        result = [updateNodesEngine((nodesToUpdate, fo))]
#    else:
#        Chunks = array_split(nodesToUpdate, p.nProc)
#        if not hasattr(p, 'pool'):
#            p.pool = Pool(processes = p.nProc)
#        #result = p.pool.imap(updateNodesEngine, [(c, fo) for c in Chunks])
#        result = p.pool.map(updateNodesEngine, [(c, fo) for c in Chunks])
#    for i, elem in enumerate(result):
#        if elem is None: continue
#        tnlh_all_new, tnlh_curr_best, M = elem
#        for j, node in enumerate(Chunks[i]): 
#            node.fo = fo
#            node.tnlh_curr = tnlh_all_new[j]
#            node.tnlh_curr_best = tnlh_curr_best[j]
#            node.th_key = M[j]



from numpy import log10, isnan
def signOfFeasible(p):
    r = '-'
    if p.isFeas(p.xk): r = '+'
    return r

textOutputDict = {\
'objFunVal': lambda p: p.iterObjFunTextFormat % (-p.Fk if p.invertObjFunc else p.Fk), 
'log10(maxResidual)': lambda p: '%0.2f' % log10(p.rk+1e-100), 
'log10(MaxResidual/ConTol)':lambda p: '%0.2f' % log10(max((p.rk/p.contol, 1e-100))), 
'residual':lambda p: '%0.1e' % p._Residual, 
'isFeasible': signOfFeasible, 
'nSolutions': lambda p: '%d' % p._nObtainedSolutions, 
'front length':lambda p: '%d' % p._frontLength, 
'outcome': lambda p: ('%+d' % -p._nOutcome if p._nOutcome != 0 else ''), 
'income': lambda p: ('%+d' % p._nIncome if p._nIncome != 0 else ''), 
'f*_distance_estim': lambda p: ('%0.1g' % p.f_bound_distance if not isnan(p.f_bound_distance) else 'N/A'), 
'f*_bound_estim': lambda p: (p.iterObjFunTextFormat % \
p.f_bound_estimation) if not isnan(p.f_bound_estimation) else 'N/A', 
}
delimiter = '  '

class ooTextOutput:
    def __init__(self):
        pass

    def iterPrint(self):

        if self.lastPrintedIter == self.iter: return

        if self.iter == 0 and self.iprint >= 0: # 0th iter (start)
            s = '  iter' + delimiter
            for fn in self.data4TextOutput:
                s += fn + delimiter
            self.disp(s)
        elif self.iprint<0 or \
        (((self.iprint>0 and self.iter % self.iprint != 0) or self.iprint==0)  and not(self.isFinished or self.iter == 0)):
            return

        s = str(self.iter).rjust(5) + '  '
        for columnName in self.data4TextOutput:
            val = textOutputDict[columnName](self)
            #nWhole = length(columnName)
            s += val.rjust(len(columnName)) + ' '
        self.disp(s)
        self.lastPrintedIter = self.iter



from openopt.kernel.baseSolver import *
#from openopt.kernel.Point import Point
#from openopt.kernel.setDefaultIterFuncs import *
from numpy import isfinite
from amsg2p import amsg2p as Solver

class amsg2p(baseSolver):
    __name__ = 'amsg2p'
    __license__ = "BSD"
    __authors__ = "Dmitrey"
    __alg__ = "Petro I. Stetsyuk, amsg2p"
    __optionalDataThatCanBeHandled__ = []
    iterfcnConnected = True
    #_canHandleScipySparse = True

    #default parameters
#    T = float64
    
    showRes = False
    show_nnan = False
    gamma = 1.0
#    approach = 'all active'

    def __init__(self): pass
    def __solver__(self, p):
        #assert self.approach == 'all active'
        if not p.isUC: p.warn('Handling of constraints is not implemented properly for the solver %s yet' % self.__name__)
        if p.fOpt is None: 
            if not isfinite(p.fEnough):
                p.err('the solver %s requires providing optimal value fOpt')
            else:
                p.warn("you haven't provided optimal value fOpt for the solver %s; fEnough = %0.2e will be used instead" %(self.__name__, p.fEnough))
                p.fOpt = p.fEnough
        if p.fTol is None: 
            s = '''
            the solver %s requires providing required objective function tolerance fTol
            15*ftol = %0.1e will be used instead
            ''' % (self.__name__, p.ftol)
            p.pWarn(s)
            fTol = 15*p.ftol
        else: fTol = p.fTol
        
        def itefcn(*args, **kwargs):
            p.iterfcn(*args, **kwargs)
            return p.istop
        x, itn = Solver(p.f, p.df, p.x0, fTol, p.fOpt, self.gamma, itefcn)

        if p.f(x) < p.fOpt + fTol:
            p.istop = 10
        #p.iterfcn(x)



__docformat__ = "restructuredtext en"
from time import time, clock, localtime
from numpy import asfarray, nan, ones, all, atleast_1d, any, isnan, \
array_equal, asscalar, asarray, ndarray, isscalar, seterr, isinf,inf#, where
from setDefaultIterFuncs import stopcase,  SMALL_DELTA_X,  SMALL_DELTA_F, IS_MAX_ITER_REACHED
from check import check
from oologfcn import OpenOptException, errSet
from openopt import __version__ as version
from result import OpenOptResult


######################
# don't change to mere ooMisc! 
from openopt.kernel.ooMisc import isSolved 
######################

#from baseProblem import ProbDefaults
from nonOptMisc import getSolverFromStringName, EmptyClass
# for PyPy
from openopt.kernel.nonOptMisc import where

try:
    import setproctitle
    hasSetproctitleModule = True
except ImportError:
    hasSetproctitleModule = False
    
try:
    from FuncDesigner import FuncDesignerException
except:
    FuncDesignerException = OpenOptException
    
#from openopt.kernel.ooMisc import __solverPaths__
ConTolMultiplier = 0.8

#if __solverPaths__ is None:
#    __solverPaths__ = {}
#    file = string.join(__file__.split(os.sep)[:-1], os.sep)
#    for root, dirs, files in os.walk(os.path.dirname(file)+os.sep+'solvers'):
#        rd = root.split(os.sep)
#        if '.svn' in rd: continue
#        rd = rd[rd.index('solvers')+1:]
#        for file in files:
#            print file
#            if len(file)>6 and file[-6:] == '_oo.py':
#                __solverPaths__[file[:-6]] = 'openopt.solvers.' + string.join(rd,'.') + '.'+file[:-3]

#import pickle
#f = open('solverPaths.py', 'w')
#solverPaths = pickle.load(f)


def runProbSolver(p_, solver_str_or_instance=None, *args, **kwargs):
    #p = copy.deepcopy(p_, memo=None, _nil=[])
    errSet.clear()
    p = p_
    p._localtime_started = localtime()
    if len(args) != 0: p.err('unexpected args for p.solve()')
    if hasattr(p, 'was_involved'): p.err("""You can't run same prob instance for twice. 
    Please reassign prob struct. 
    You can avoid it via using FuncDesigner oosystem.""")
    else: p.was_involved = True

    if solver_str_or_instance is None:
        if hasattr(p, 'solver'): solver_str_or_instance = p.solver
        elif 'solver' in kwargs: solver_str_or_instance = kwargs['solver']

    if type(solver_str_or_instance) is str and ':' in solver_str_or_instance:
        isConverter = True
        probTypeToConvert,  solverName = solver_str_or_instance.split(':', 1)
        p.solver = getSolverFromStringName(p, solverName)
        solver_params = {}
        #return converter(solverName, *args, **kwargs)
    else:
        isConverter = False
        if solver_str_or_instance is None:
            p.err('you should provide name of solver')
        elif type(solver_str_or_instance) is str:
            p.solver = getSolverFromStringName(p, solver_str_or_instance)
        else: # solver_str_or_instance is oosolver
            if not solver_str_or_instance.isInstalled:
                p.err('''
                solver %s seems to be uninstalled yet, 
                check http://openopt.org/%s for install instructions''' % (solver_str_or_instance.__name__, p.probType))
            p.solver = solver_str_or_instance
            for key, value  in solver_str_or_instance.fieldsForProbInstance.items():
                setattr(p, key, value)
    p.isConverterInvolved = isConverter
    if isConverter:
        p.err('converters are temporarily disabled, use native solvers')

    old_err = seterr(all= 'ignore')
    
    if 'debug' in kwargs.keys():
       p.debug =  kwargs['debug']

    probAttributes = set(p.__dict__)
    solverAttributes = set(p.solver.__dict__)
    intersection = list(probAttributes.intersection(solverAttributes))
    if len(intersection) != 0:
        if p.debug:
            p.warn('''
            attribute %s is present in both solver and prob 
            (probably you assigned solver parameter in prob constructor), 
            the attribute will be assigned to solver''' % intersection[0])
        for elem in intersection:
            setattr(p.solver, elem, getattr(p, elem))

    solver = p.solver.__solver__
    
    for key, value in kwargs.items():
        if hasattr(p.solver, key):
            if isConverter:
                solver_params[key] = value
            else:
                setattr(p.solver, key, value)
        elif hasattr(p, key) or key in ('lb', 'ub'):
            setattr(p, key, value)
        else: 
            p.warn('incorrect parameter for prob.solve(): "' + str(key) + '" - will be ignored (this one has been not found in neither prob nor ' + p.solver.__name__ + ' solver parameters)')
    if p.probType == 'EIG' and 'goal' in kwargs:
        p.err('for EIG parameter "goal" should be used only in class instance definition, not in "solve" method')
        
    setSomeDefaults(p)
    
    T = time()
    C = clock()
    try:
        p._Prepare()
    except OpenOptException as ooe:
        p.err(ooe.msg)
    except FuncDesignerException as fde:
        p.err(fde.msg)
    
    p.initTime = time() - T
    p.initCPUTime = clock() - C
    if p.initTime > 1 or p.initCPUTime > 1:
        p.disp('Initialization: Time = %0.1f CPUTime = %0.1f' % (p.initTime, p.initCPUTime))
        
    for fn in ['FunEvals', 'Iter', 'Time', 'CPUTime']:
        if hasattr(p,'min'+fn) and hasattr(p,'max'+fn) and getattr(p,'max'+fn) < getattr(p,'min'+fn):
            p.warn('min' + fn + ' (' + str(getattr(p,'min'+fn)) +') exceeds ' + 'max' + fn + '(' + str(getattr(p,'max'+fn)) +'), setting latter to former')
            setattr(p,'max'+fn, getattr(p,'min'+fn))

    for fn in ['maxFunEvals', 'maxIter']: setattr(p, fn, int(getattr(p, fn)))# to prevent warnings from numbers like 1e7

    if hasattr(p, 'x0'): 
        try:
            p.x0 = atleast_1d(asfarray(p.x0).copy())
        except NotImplementedError:
            p.x0 = asfarray(p.x0.tolist())
    for fn in ['lb', 'ub', 'b', 'beq']:
        if hasattr(p, fn):
            fv = getattr(p, fn)
            if fv is not None:# and fv != []:
                if str(type(fv)) == "<class 'map'>":
                    p.err("Python3 incompatibility with previous versions: you can't use 'map' here, use rendered value instead")
                setattr(p, fn, asfarray(fv).flatten())
            else:
                setattr(p, fn, asfarray([]))

    if p.solver._requiresFiniteBoxBounds:
        ind1, ind2 = isinf(p.lb), isinf(p.ub)
        if isscalar(p.implicitBounds): p.implicitBounds = (-p.implicitBounds, p.implicitBounds) # may be from lp2nlp converter, thus omit nlp init code
        p.lb[ind1] = p.implicitBounds[0] if asarray(p.implicitBounds[0]).size == 1 else p.implicitBounds[0][ind1]
        p.ub[ind2] = p.implicitBounds[1] if asarray(p.implicitBounds[1]).size == 1 else p.implicitBounds[0][ind2]


#    if p.lb.size == 0:
#        p.lb = -inf * ones(p.n)
#    if p.ub.size == 0:
#        p.ub = inf * ones(p.n)

    p.stopdict = {}

    for s in ['b','beq']:
        if hasattr(p, s): setattr(p, 'n'+s, len(getattr(p, s)))

    #if p.probType not in ['LP', 'QP', 'MILP', 'LLSP']: p.objFunc(p.x0)

    p.isUC = p._isUnconstrained()
    
    isIterPointAlwaysFeasible = p.solver.__isIterPointAlwaysFeasible__ if type(p.solver.__isIterPointAlwaysFeasible__) == bool \
        else p.solver.__isIterPointAlwaysFeasible__(p)
    if isIterPointAlwaysFeasible:
        #assert p.data4TextOutput[-1] == 'log10(maxResidual)'
        if p.data4TextOutput[-1] == 'log10(maxResidual)': 
            p.data4TextOutput = p.data4TextOutput[:-1]
#        else:
#            p.err('bug in runProbSolver.py')
    elif p.useScaledResidualOutput:
        p.data4TextOutput[-1] = 'log10(MaxResidual/ConTol)'

    if p.showFeas and p.data4TextOutput[-1] != 'isFeasible': 
        p.data4TextOutput.append('isFeasible')
    # TODO: rework
    if p.solver.__name__ == 'interalg' and p.isObjFunValueASingleNumber:
        p.data4TextOutput = p.data4TextOutput[:1] + \
        ['f*_distance_estim', 'f*_bound_estim'] + p.data4TextOutput[1:]
    if p.maxSolutions != 1:
        p._nObtainedSolutions = 0
        p.data4TextOutput.append('nSolutions')

    if not p.solver.iterfcnConnected:
        if SMALL_DELTA_X in p.kernelIterFuncs: p.kernelIterFuncs.pop(SMALL_DELTA_X)
        if SMALL_DELTA_F in p.kernelIterFuncs: p.kernelIterFuncs.pop(SMALL_DELTA_F)

    if not p.solver._canHandleScipySparse:
        if hasattr(p.A, 'toarray'): p.A = p.A.toarray()
        if hasattr(p.Aeq, 'toarray'): p.Aeq = p.Aeq.toarray()
    
    if isinstance(p.A, ndarray) and type(p.A) != ndarray: # numpy matrix
        p.A = p.A.A 
    if isinstance(p.Aeq, ndarray) and type(p.Aeq) != ndarray: # numpy matrix
        p.Aeq = p.Aeq.A 

    if hasattr(p, 'optVars'):
        p.err('"optVars" is deprecated, use "freeVars" instead ("optVars" is not appropriate for some prob types, e.g. systems of (non)linear equations)')

#    p.xf = nan * ones([p.n, 1])
#    p.ff = nan
    #todo : add scaling, etc
    p.primalConTol = p.contol
    if not p.solver.__name__.startswith('interalg'): 
        p.contol *= ConTolMultiplier

    p.timeStart = time()
    p.cpuTimeStart = clock()
    
    # TODO: move it into solver parameters
    if p.probType not in ('MINLP', 'IP'):
        p.plotOnlyCurrentMinimum = p.__isNoMoreThanBoxBounded__()


    ############################
    # Start solving problem:

    if p.iprint >= 0:
        p.disp('\n' + '-'*25 + ' OpenOpt %s ' % version + '-'*25)
        pt = p.probType if p.probType != 'NLSP' else 'SNLE'
        s = 'problem: ' + p.name + '   type: %s' % pt
        if p.showGoal: 
            s += '    goal: %s' % p.goal
        p.disp(s)
        s = 'solver: ' +  p.solver.__name__ 
        if p.solver.__name__ == 'interalg':
            if p.isObjFunValueASingleNumber:
                if p.fTol is None and p.probType in ('IP', 'ODE'):
                    p.pWarn('''
                    please use p.fTol parameter for interalg with this problem;
                    p.ftol currently will be used instead
                    ''')
                    p.fTol = p.ftol
                if p.fTol is None:
                    p.fTol = 1e-7
                    p.warn('''
                    interalg requires p.fTol value (required objective function tolerance); 
                    10^-7 will be used'''
                    )
                s += '   fTol: %0.1g ' % p.fTol 
                if p.probType != 'IP':# TODO: implement for IP
                    s += '   rTol: %0.1g ' % p.rTol 
            elif p.probType in ('SNLE', 'NLSP'):
                s += '   maxSolutions: %s ' % p.maxSolutions 
        p.disp(s)

    p.extras = {}

    
    try:
#        if 0 and isConverter:
#            pass
#            # TODO: will R be somewhere used?
#            #R = converter(solverName, **solver_params)
#        else:
        nErr = check(p)
        if nErr: p.err("prob check results: " +str(nErr) + "ERRORS!")#however, I guess this line will be never reached.
        if p.probType not in ('IP', 'EIG'): p.iterfcn(p.x0)
        originalName = setProcTitle(p)
        
#        import os, cProfile
#        print '----'
#        print 'p.n, p.nb, p.nbeq, p.nc, p.nh:',  p.n, p.nb, p.nbeq, p.nc, p.nh
#        if p.solver.__name__ == 'interalg':
#            cProfile.runctx( 'solver(p)', globals(), locals(), filename="t.profile" )
#        #os.system('runsnake t.profile')
#        else:
#            solver(p)
            
        solver(p)
        
#        if p.solver.__name__ == 'interalg':
#            
#            print p.extras
#            os.system('pyprof2calltree -i t.profile -o myfile.prof.grind')
#            os.system('kcachegrind myfile.prof.grind')
#        print p.xk
#        print '===='

#    except killThread:
#        if p.plot:
#            print 'exiting pylab'
#            import pylab
#            if hasattr(p, 'figure'):
#                print 'closing figure'
#                #p.figure.canvas.draw_drawable = lambda: None
#                pylab.ioff()
#                pylab.close()
#                #pylab.draw()
#            #pylab.close()
#            print 'pylab exited'
#        return None
    except isSolved:
#        p.fk = p.f(p.xk)
#        p.xf = p.xk
#        p.ff = p.objFuncMultiple2Single(p.fk)

        if p.istop == 0: p.istop = 1000
    except OpenOptException as ooe:
        p.istop = -1
        p.err(ooe.msg)
    except FuncDesignerException as fde:
        p.err(fde.msg)
    finally:
        if p.isFDmodel:
            for v in p.freeVarsSet | p.fixedVarsSet:
                if v.fields != ():
                    v.domain, v.aux_domain = v.aux_domain, v.domain
        seterr(**old_err)

    if hasSetproctitleModule and originalName is not None:
        setproctitle.setproctitle(originalName)
    if len(errSet) != 0:
        finalGUIroutines(p)
        p._immutable = True
        return None
    ############################
    p.contol = p.primalConTol

    # Solving finished
    if hasattr(p, '_bestPoint') and not any(isnan(p._bestPoint.x)) and p.probType != 'ODE':
        try:
            p.iterfcn(p._bestPoint)
        except isSolved:
            pass
            
    if p.probType != 'EIG':
        if not hasattr(p, 'xf') and not hasattr(p, 'xk'): 
            p.xf = p.xk = ones(p.n)*nan
        if hasattr(p, 'xf') and (not hasattr(p, 'xk') or array_equal(p.xk, p.x0)): 
            p.xk = p.xf
        if not hasattr(p,  'xf') or all(isnan(p.xf)): 
            p.xf = p.xk
        if p.xf is nan: 
            p.xf = p.xk = ones(p.n)*nan
        
        if p.isFeas(p.xf) and (not p.probType=='MINLP' or p.discreteConstraintsAreSatisfied(p.xf)):
            p.isFeasible = True
        else: p.isFeasible = False
    else:
        p.isFeasible = True # check it!
    
    p.isFinished = True # After the feasibility check above!
    p._localtime_finished = localtime()
    
    if p.probType == 'MOP':
        p.isFeasible = True
    elif p.probType == 'IP':
        p.isFeasible = p.rk < p.ftol
    else:
        p.ff = p.fk = p.objFunc(p.xk)
        
        # walkaround for PyPy:
        if type(p.ff) == ndarray and p.ff.size == 1:
            p.ff = p.fk = asscalar(p.ff)
        
    if not hasattr(p,  'ff') or any(isnan(p.ff)): 
        p.iterfcn, tmp_iterfcn = lambda *args: None, p.iterfcn
        p.ff = p.fk
        p.iterfcn = tmp_iterfcn

    if p.invertObjFunc:  
        p.fk, p.ff = -p.fk, -p.ff

    if asfarray(p.ff).size > 1: 
        p.ff = p.objFuncMultiple2Single(p.fk)

    #p.ff = p.objFuncMultiple2Single(p.ff)
    #if not hasattr(p, 'xf'): p.xf = p.xk
    if type(p.xf) in (list, tuple) or isscalar(p.xf): 
        p.xf = asarray(p.xf)
    p.xf = p.xf.flatten()
    p.rf = p.getMaxResidual(p.xf) if p.probType not in ('IP', 'ODE') else p.rk

    if not p.isFeasible and p.istop > 0: p.istop = -100-p.istop/1000.0
    
    if p.istop == 0 and p.iter >= p.maxIter:
        p.istop, p.msg = IS_MAX_ITER_REACHED, 'Max Iter has been reached'
    
    p.stopcase = stopcase(p)

    p.xk, p.rk = p.xf, p.rf
    if p.invertObjFunc: 
        p.fk = -p.ff
        p.iterfcn(p.xf, -p.ff, p.rf)
    else: 
        p.fk = p.ff
        p.iterfcn(p.xf, p.ff, p.rf)
        
    p.lb, p.ub = p._lb, p._ub
    checkImplicitBounds(p)

    p.__finalize__()
    if not p.storeIterPoints: delattr(p.iterValues, 'x')

    r = OpenOptResult(p)

    #TODO: add scaling handling!!!!!!!
#    for fn in ('df', 'dc', 'dh', 'd2f', 'd2c', 'd2h'):
#        if hasattr(p, '_' + fn): setattr(r, fn, getattr(p, '_'+fn))

    p.invertObjFunc = False
    
    if p.isFDmodel:
        p.x0 = p._x0

    finalTextOutput(p, r)
    finalGUIroutines(p)
    
    finish = p.finish if type(p.finish) in (list, tuple, set) else [p.finish]
    for f in finish:
        f(r, p)
    p._immutable = True
    
    return r
    
##########################################################
def finalGUIroutines(p):
    if p.isManagerUsed: 
        p.GUI_items['Quit'].config(state='active')
        p.GUI_items['time'].set('%d' % (time() - p.timeStart))
        p.GUI_items['cputime'].set('%d' % (clock() - p.cpuTimeStart))
    elif p.plot:
        finalShow(p)
        
##########################################################
def finalTextOutput(p, r):
    if type(p.msg) == ndarray: # from fmincon etc
        msg = asarray(p.msg.flatten(),int).tolist()
        p.msg = ''.join(chr(c) for c in msg)
    if p.msg == '':
        msg = 'finished'
    if p.iprint >= 0:
        if len(p.msg):  
            p.disp("istop: " + str(r.istop) + ' (' + p.msg +')')
        else: 
            p.disp("istop: " + str(r.istop))

        p.disp('Solver:   Time Elapsed = ' + str(r.elapsed['solver_time']) + ' \tCPU Time Elapsed = ' + str(r.elapsed['solver_cputime']))
        if p.plot:
            p.disp('Plotting: Time Elapsed = '+ str(r.elapsed['plot_time'])+ ' \tCPU Time Elapsed = ' + str(r.elapsed['plot_cputime']))
        
        if p.probType == 'MOP':
            msg = '%d solutions have been obtained' % len(p.solutions.coords)
            p.disp(msg)
            return
        
        # TODO: add output of NaNs number in constraints (if presernt)
        rMsg = 'max(residuals/requiredTolerances) = %g' % \
        (r.rf / p.contol) if p.useScaledResidualOutput else 'MaxResidual = %g' % r.rf

        if not p.isFeasible:
            nNaNs = (len(where(isnan(p.c(p.xf)))[0]) if p.c is not None and type(p.c)!=list else 0) \
            + (len(where(isnan(p.h(p.xf)))[0]) if p.h is not None and type(p.h)!=list else 0)
            if nNaNs == 0:
                nNaNsMsg = ''
            elif nNaNs == 1:
                nNaNsMsg = '1 constraint is equal to NaN, '
            else:
                nNaNsMsg = ('%d constraints are equal to NaN, ' % nNaNs)
            p.disp('NO FEASIBLE SOLUTION has been obtained (%s%s, objFunc = %0.8g)' % (nNaNsMsg,  rMsg, r.ff))
        else:
            if p.maxSolutions == 1:
                msg = "objFuncValue: " + (p.finalObjFunTextFormat % r.ff)
                if not p.isUC: msg += ' (feasible, %s)' % rMsg
            else:
                msg = '%d solutions have been obtained' % len(p.solutions)
            p.disp(msg)

##########################################################
def finalShow(p):
    if not p.plot: return
    pylab = __import__('pylab')
    pylab.ioff()
    if p.show:
#        import os
#        if os.fork():
            pylab.show()

##########################################################
def checkImplicitBounds(p):
    if p.solver._requiresFiniteBoxBounds and p.implicitBounds is not None:
        delta = 0.01 * (p.implicitBounds[1] - p.implicitBounds[0])
        tmp = where(isinf(p.lb), p.implicitBounds[0], -inf)
        delta_r = p.xk - tmp
        tmp = where(isinf(p.ub), p.implicitBounds[1], inf)
        delta_l = tmp - p.xk
        if any(delta_r<delta) or any(delta_l<delta):
            Msg = '''
            some coordinates of solution are too close to implicitBounds
            (difference is less than 1%)
            '''
            p.warn(Msg)
            if p.istop > 0:
                p.istop = p.stopcase = 0 
                p.msg += ';' + Msg

##########################################################
def setProcTitle(p):
    originalName = None
    if hasSetproctitleModule:
        try:
            originalName = setproctitle.getproctitle()
            if originalName.startswith('OpenOpt-'):
                originalName = None
            else:
                s = 'OpenOpt-' + p.solver.__name__
                # if p.name != 'unnamed':
                s += '-' + p.name
                setproctitle.setproctitle(s)
        except:
            pass
    else:
        p.pWarn('''
        please install setproctitle module 
        (it's available via easy_install and Linux soft channels like apt-get)''')
    return originalName

##########################################################
def setSomeDefaults(p):
    p.iterValues = EmptyClass()
    p.iterCPUTime = []
    p.iterTime = []
    p.iterValues.x = [] # iter points
    p.iterValues.f = [] # iter ObjFunc Values
    p.iterValues.r = [] # iter MaxResidual
    p.iterValues.rt = [] # iter MaxResidual Type: 'c', 'h', 'lb' etc
    p.iterValues.ri = [] # iter MaxResidual Index
    p.solutions = [] # list of solutions, may contain several elements for interalg and mb other solvers
    if p._baseClassName == 'NonLin':
        p.iterValues.nNaNs = [] # number of constraints equal to numpy.nan

    if p.goal in ['max','maximum']: p.invertObjFunc = True

    #TODO: remove it!
    p.advanced = EmptyClass()

    p.istop = 0
    p.iter = 0
    p.graphics.nPointsPlotted = 0
    p.finalIterFcnFinished = False
    #for fn in p.nEvals.keys(): p.nEvals[fn] = 0 # NB! f num is used in LP/QP/MILP/etc stop criteria check

    p.msg = ''
    if not type(p.callback) in (list,  tuple): 
        p.callback = [p.callback]
    if hasattr(p, 'xlabel'): 
        p.graphics.xlabel = p.xlabel
    if p.graphics.xlabel == 'nf': 
        p.iterValues.nf = [] # iter ObjFunc evaluation number




"""
Solving system of equations:
x[0]**3+x[1]**3-9 = 0
x[0]-0.5*x[1] = 0
cos(x[2])+x[0]-1.5 = 0
"""

from openopt import SNLE
from numpy import asfarray, zeros, cos, sin

#f = lambda x: (x[0]**3+x[1]**3-9, x[0]-0.5*x[1], cos(x[2])+x[0]-1.5)
# or:
f = (lambda x: x[0]**3+x[1]**3-9, lambda x: x[0]-0.5*x[1], lambda x: cos(x[2])+x[0]-1.5)
# Python list, numpy.array are allowed as well:
#f = lambda x: [x[0]**3+x[1]**3-9, x[0]-0.5*x[1], cos(x[2])+x[0]-1.5]
#or f = lambda x: asfarray((x[0]**3+x[1]**3-9, x[0]-0.5*x[1], cos(x[2])+x[0]-1.5))

# start point
x0 = [8,15, 80]

#optional: gradient
df = (lambda x: [3*x[0]**2, 3*x[1]**2,  0], lambda x: [1, -0.5,  0], lambda x:[1, 0, -sin(x[2])])

#w/o gradient:
#p = SNLE(f, x0)
p = SNLE(f, x0, df = df)

#optional: user-supplied gradient check:
#p.checkdf()

#optional: graphical output, requires matplotlib installed
p.plot = 1

#r = p.solve('scipy_fsolve')
r = p.solve('nssolve')
#or using converter to nlp, try to minimize sum(f_i(x)^2):
#r = p.solve('nlp:ralg')

print('solution: %s' % r.xf)
print('max residual: %e' % r.ff)
###############################
#should print:
#solution: [  1.           2.          55.50147021] (3rd coord may differ due to cos is periodic)
#max residual: 2.72366951215e-09



__docformat__ = "restructuredtext en"
from numpy import *
from openopt.kernel.baseSolver import baseSolver
import pywrapper
from openopt.kernel.setDefaultIterFuncs import SMALL_DF
#from openopt.kernel.ooMisc import isSolved

class algencan(baseSolver):
    __name__ = 'algencan'
    __license__ = "GPL"
    __authors__ = 'J. M. Martinez martinezimecc@gmail.com, Ernesto G. Birgin egbirgin@ime.usp.br, Jan Marcel Paiva Gentil jgmarcel@ime.usp.br'
    __alg__ = "Augmented Lagrangian Multipliers"
    __homepage__ = 'http://www.ime.usp.br/~egbirgin/tango/'
    #TODO: edit info!
    __info__ = "please pay more attention to gtol param, it's the only one ALGENCAN positive stop criterium, xtol and ftol are unused"
    __optionalDataThatCanBeHandled__ = ['A', 'Aeq', 'b', 'beq', 'lb', 'ub', 'c', 'h']
    __isIterPointAlwaysFeasible__ = lambda self, p: p.__isNoMoreThanBoxBounded__()

    def __init__(self): pass
    def __solver__(self, p):
        def inip():
            """This subroutine must set some problem data.

            For achieving this objective YOU MUST MODIFY it according to your
            problem. See below where your modifications must be inserted.

            Parameters of the subroutine:

            On Entry:

            This subroutine has no input parameters.

            On Return

            n        integer,
                     number of variables,

            x        double precision x(n),
                     initial point,

            l        double precision l(n),
                     lower bounds on x,

            u        double precision u(n),
                     upper bounds on x,

            m        integer,
                     number of constraints (excluding the bounds),

            lambda   double precision lambda(m),
                     initial estimation of the Lagrange multipliers,

            equatn   logical equatn(m)
                     for each constraint j, set equatn(j) = .true. if it is an
                     equality constraint of the form c_j(x) = 0, and set
                     equatn(j) = .false. if it is an inequality constraint of
                     the form c_j(x) <= 0,

            linear   logical linear(m)
                     for each constraint j, set linear(j) = .true. if it is a
                     linear constraint, and set linear(j) = .false. if it is a
                     nonlinear constraint.
            """

            #   Number of variables

            n = p.n

            #   Number of constraints (equalities plus inequalities)
#            if p.userProvided.c:  nc = p.c(p.x0).size
#            else: nc = 0
#            if p.userProvided.h:  nh = p.h(p.x0).size
#            else: nh = 0
            nc,  nh = p.nc, p.nh
            nb, nbeq = p.b.size, p.beq.size
            #p.algencan.nc, p.algencan.nh, p.algencan.nb, p.algencan.nbeq = nc, nh, nb, nbeq

            m = nc + nh + nb + nbeq

            #   Initial point

            x = p.x0

            #   Lower and upper bounds

            l = p.lb
            l[l<-1.0e20] = -1.0e20

            u = p.ub
            u[u>1.0e20] = 1.0e20

            #   Lagrange multipliers approximation. Most users prefer to use the
            #   null initial Lagrange multipliers estimates. However, if the
            #   problem that you are solving is "slightly different" from a
            #   previously solved problem of which you know the correct Lagrange
            #   multipliers, we encourage you to set these multipliers as initial
            #   estimates. Of course, in this case you are also encouraged to use
            #   the solution of the previous problem as initial estimate of the
            #   solution. Similarly, most users prefer to use rho = 10 as initial
            #   penalty parameters. But in the case mentioned above (good
            #   estimates of solution and Lagrange multipliers) larger values of
            #   the penalty parameters (say, rho = 1000) may be more useful. More
            #   warm-start procedures are being elaborated.

            lambda_ = zeros(m)

            #   For each constraint i, set equatn[i] = 1. if it is an equality
            #   constraint of the form c_i(x) = 0, and set equatn[i] = 0 if
            #   it is an inequality constraint of the form c_i(x) <= 0.


            equatn = array( [False]*nc + [True]*nh + [False]*nb + [True]*nbeq)



            #   For each constraint i, set linear[i] = 1 if it is a linear
            #   constraint, otherwise set linear[i] = 0.

            linear = array( [False]*nc + [False]*nh + [True]*nb + [True]*nbeq)


            coded = [True,  # evalf
                     True,  # evalg
                     p.userProvided.d2f,  # evalh
                     True,  # evalc
                     True,  # evaljac
                     False,  # evalhc
                     False, # evalfc
                     False, # evalgjac
                     False, # evalhl
                     False] # evalhlp

            checkder = False
            #checkder = 1

            return n,x,l,u,m,lambda_,equatn.tolist(),linear.tolist(),coded,checkder

        #   ******************************************************************
        #   ******************************************************************

        def evalf(x):
            """This subroutine must compute the objective function.

            For achieving this objective YOU MUST MODIFY it according to your
            problem. See below where your modifications must be inserted.

            Parameters of the subroutine:

            On Entry:

            x        double precision x(n),
                     current point,

            On Return

            f        double precision,
                     objective function value at x,

            flag     integer,
                     You must set it to any number different of 0 (zero) if
                     some error ocurred during the evaluation of the objective
                     function. (For example, trying to compute the square root
                     of a negative number, dividing by zero or a very small
                     number, etc.) If everything was o.k. you must set it
                     equal to zero.
            """

            f = p.f(x)
            #if f is None: raise isSolved
            if f is not nan: flag = 0
            else: flag = 1

            return f,flag

        #   ******************************************************************
        #   ******************************************************************

        def evalg(x):
            """This subroutine must compute the gradient vector of the objective \
        function.

            For achieving these objective YOU MUST MODIFY it in the way specified
            below. However, if you decide to use numerical derivatives (we dont
            encourage this option at all!) you dont need to modify evalg.

            Parameters of the subroutine:

            On Entry:

            x        double precision x(n),
                     current point,

            On Return

            g        double precision g(n),
                     gradient vector of the objective function evaluated at x,

            flag     integer,
                     You must set it to any number different of 0 (zero) if
                     some error ocurred during the evaluation of any component
                     of the gradient vector. (For example, trying to compute
                     the square root of a negative number, dividing by zero or
                     a very small number, etc.) If everything was o.k. you
                     must set it equal to zero.
            """

            g = p.df(x)
            #if g is None: raise isSolved
            if any(isnan(g)): flag = 1
            else: flag = 0

            return g,flag

        #   ******************************************************************
        #   ******************************************************************

        def evalh(x):
#            """This subroutine might compute the Hessian matrix of the objective \
#        function.
#
#            For achieving this objective YOU MAY MODIFY it according to your
#            problem. To modify this subroutine IS NOT MANDATORY. See below
#            where your modifications must be inserted.
#
#            Parameters of the subroutine:
#
#            On Entry:
#
#            x        double precision x(n),
#                     current point,
#
#            On Return
#
#            nnzh     integer,
#                     number of perhaps-non-null elements of the computed
#                     Hessian,
#
#            hlin     integer hlin(nnzh),
#                     see below,
#
#            hcol     integer hcol(nnzh),
#                     see below,
#
#            hval     double precision hval(nnzh),
#                     the non-null value of the (hlin(k),hcol(k)) position
#                     of the Hessian matrix of the objective function must
#                     be saved at hval(k). Just the lower triangular part of
#                     Hessian matrix must be computed,
#
#            flag     integer,
#                     You must set it to any number different of 0 (zero) if
#                     some error ocurred during the evaluation of the Hessian
#                     matrix of the objective funtion. (For example, trying
#                     to compute the square root of a negative number,
#                     dividing by zero or a very small number, etc.) If
#                     everything was o.k. you must set it equal to zero.
#            """
#
            flag = 0
#            if hasattr(self, 'd2f'):
#                return self.d2f

            try:
                H = p.d2f(x)
            except:
                nnzh = 0
                hlin = zeros(nnzh, int)
                hcol = zeros(nnzh, int)
                hval = zeros(nnzh, float)
                flag = 1
                return hlin,hcol,hval,nnzh,flag

            ind = H.nonzero()
            (ind_0, ind_1) = ind
            ind_greater = ind_0 >= ind_1
            ind_0, ind_1 = ind_0[ind_greater], ind_1[ind_greater]

            nnzh = ind_0.size
            val = H[(ind_0, ind_1)]
            hlin, hcol, hval = ind_0, ind_1, val

#            if lower(p.castFrom) in ('QP', 'LLSP'):
#                self.d2f = hlin,hcol,hval,nnzh,flag

            return hlin,hcol,hval,nnzh,flag

        #   ******************************************************************
        #   ******************************************************************

        def evalc(x,ind):
            """This subroutine must compute the ind-th constraint.

            For achieving this objective YOU MUST MOFIFY it according to your
            problem. See below the places where your modifications must be
            inserted.

            Parameters of the subroutine:

            On Entry:

            x        double precision x(n),
                     current point,

            ind      integer,
                     index of the constraint to be computed,

            On Return

            c        double precision,
                     i-th constraint evaluated at x,

            flag     integer
                     You must set it to any number different of 0 (zero) if
                     some error ocurred during the evaluation of the
                     constraint. (For example, trying to compute the square
                     root of a negative number, dividing by zero or a very
                     small number, etc.) If everything was o.k. you must set
                     it equal to zero.
            """
            flag = 0

            #TODO: recalculate i-th constraint, not all

            i = ind - 1 # Python enumeration starts from 0, not 1

            if i < p.nc:
                c = p.c(x, i)
            elif p.nc <= i < p.nc + p.nh:
                c = p.h(x, i-p.nc)
            elif p.nc + p.nh <= i < p.nc + p.nh + p.nb:
                j = i - p.nc - p.nh
                c = p.dotmult(p.A[j], x).sum() - p.b[j]
            elif i < p.nc + p.nh + p.nb + p.nbeq:
                j = i - p.nc - p.nh - p.nb
                c = p.dotmult(p.Aeq[j], x).sum() - p.beq[j]
            else:
                flag = -1
                p.err('error in connection algencan to openopt')

            if any(isnan(c)): flag = -1
            #if c is None: raise isSolved
            return c,flag

        #   ******************************************************************
        #   ******************************************************************

        def evaljac(x,ind):
            """This subroutine must compute the gradient of the ind-th constraint.

            For achieving these objective YOU MUST MODIFY it in the way specified
            below.

            Parameters of the subroutine:

            On Entry:

            x        double precision x(n),
                     current point,

            ind      integer,
                     index of the constraint whose gradient will be computed,

            On Return

            nnzjac   integer,
                     number of perhaps-non-null elements of the computed
                     gradient,

            indjac   integer indjac(nnzjac),
                     see below,

            valjac   double precision valjac(nnzjac),
                     the non-null value of the partial derivative of the i-th
                     constraint with respect to the indjac(k)-th variable must
                     be saved at valjac(k).

            flag     integer
                     You must set it to any number different of 0 (zero) if
                     some error ocurred during the evaluation of the
                     constraint. (For example, trying to compute the square
                     root of a negative number, dividing by zero or a very
                     small number, etc.) If everything was o.k. you must set
                     it equal to zero.
            """


            flag = 0

            #TODO: recalculate i-th constraint, not all

            i = ind - 1 # Python enumeration starts from 0, not 1

            if i < p.nc:
                dc = p.dc(x, i)
            elif p.nc <= i < p.nc + p.nh:
                dc = p.dh(x, i-p.nc)
            elif p.nc + p.nh <= i < p.nc + p.nh + p.nb:
                j = i - p.nc - p.nh
                dc = p.A[j]
            elif i < p.nc + p.nh + p.nb + p.nbeq:
                j = i - p.nc - p.nh - p.nb
                dc = p.Aeq[j]
            else:
                p.err('error in connection algencan to openopt')
            #if dc is None: raise isSolved
            dc = dc.flatten()

            if any(isnan(dc)):
                flag = -1
                if p.debug: p.warn('algencan: nan in jacobian')

            indjac, = dc.nonzero()
            valjac = dc[indjac]
            nnzjac = indjac.size

            return indjac,valjac,nnzjac,flag

        #   ******************************************************************
        #   ******************************************************************

        def evalhc(x,ind):
            pass
#            """This subroutine might compute the Hessian matrix of the ind-th \
#        constraint.
#
#            For achieving this objective YOU MAY MODIFY it according to your
#            problem. To modify this subroutine IS NOT MANDATORY. See below
#            where your modifications must be inserted.
#
#            Parameters of the subroutine:
#
#            On Entry:
#
#            x        double precision x(n),
#                     current point,
#
#            ind      integer,
#                     index of the constraint whose Hessian will be computed,
#
#            On Return
#
#            nnzhc    integer,
#                     number of perhaps-non-null elements of the computed
#                     Hessian,
#
#            hclin    integer hclin(nnzhc),
#                     see below,
#
#            hccol    integer hccol(nnzhc),
#                     see below,
#
#            hcval    double precision hcval(nnzhc),
#                     the non-null value of the (hclin(k),hccol(k)) position
#                     of the Hessian matrix of the ind-th constraint must
#                     be saved at hcval(k). Just the lower triangular part of
#                     Hessian matrix must be computed,
#
#            flag     integer,
#                     You must set it to any number different of 0 (zero) if
#                     some error ocurred during the evaluation of the Hessian
#                     matrix of the ind-th constraint. (For example, trying
#                     to compute the square root of a negative number,
#                     dividing by zero or a very small number, etc.) If
#                     everything was o.k. you must set it equal to zero.
#            """
#
#            n = len(x)
#
#            nnzhc = 1
#
#            hclin = zeros(nnzhc, int)
#            hccol = zeros(nnzhc, int)
#            hcval = zeros(nnzhc, float)
#
#            if ind == 1:
#                hclin[0] = 0
#                hccol[0] = 0
#                hcval[0] = 2.0
#
#                flag = 0
#
#            elif ind == 2:
#                nnzhc = 0
#
#                flag = 0
#
#            else:
#                flag = -1
#
#            return hclin,hccol,hcval,nnzhc,flag

        #   ******************************************************************
        #   ******************************************************************

        def evalhlp(x,m,lambda_,p,goth):
            pass
            """This subroutine computes the product of the Hessian of the Lagrangian \
        times a vector.

              This subroutine might compute the product of the Hessian of the
              Lagrangian times vector p (just the Hessian of the objective
              function in the unconstrained or bound-constrained case).

              Parameters of the subroutine:

              On Entry:

              x        double precision x(n),
                       current point,

              m        integer,
                       number of constraints,

              lambda   double precision lambda(m),
                       vector of Lagrange multipliers,

              p        double precision p(n),
                       vector of the matrix-vector product,

              goth     logical,
                       can be used to indicate if the Hessian matrices were
                       computed at the current point. It is set to .false.
                       by the optimization method every time the current
                       point is modified. Sugestion: if its value is .false.
                       then compute the Hessians, save them in a common
                       structure and set goth to .true.. Otherwise, just use
                       the Hessians saved in the common block structure,

              On Return

              hp       double precision hp(n),
                       Hessian-vector product,

              goth     logical,
                       see above,

              flag     integer,
                       You must set it to any number different of 0 (zero) if
                       some error ocurred during the evaluation of the
                       Hessian-vector product. (For example, trying to compute
                       the square root of a negative number, dividing by zero
                       or a very small number, etc.) If everything was o.k. you
                       must set it equal to zero.
            """

#            n = len(x)
#
#            hp = zeros(n)
#
#            flag = -1
#
#            return hp,goth,flag

        #   ******************************************************************
        #   ******************************************************************

        def evalfc(*args, **kwargs):
            pass

        def evalgjac(*args, **kwargs):
            pass

        def evalhl(*args, **kwargs):
            pass

        def endp(x,l,u,m,lambda_,equatn,linear):
            """This subroutine can be used to do some extra job.

            This subroutine can be used to do some extra job after the solver
            has found the solution, like some extra statistics, or to save the
            solution in some special format or to draw some graphical
            representation of the solution. If the information given by the
            solver is enough for you then leave the body of this subroutine
            empty.

            Parameters of the subroutine:

            The parameters of this subroutine are the same parameters of
            subroutine inip. But in this subroutine there are not output
            parameter. All the parameters are input parameters.
            """
            p.xk = x.copy()
            #p.fk = p.f(x)
            #p.xf = x.copy()
            #p.ff = p.fk
            #p.iterfcn()

        ###########################################################################
        # solver body

        param = {'epsfeas': p.contol,'epsopt' : p.gtol,'iprint': 0, 'ncomp':5,'maxtotit' : p.maxIter, 'maxtotfc': p.maxFunEvals}
        pywrapper.solver(evalf,evalg,evalh,evalc,evaljac,evalhc,evalfc,evalgjac,evalhl, evalhlp,inip,endp,param)

        if p.istop == 0:
            p.istop = SMALL_DF
            p.msg = '|| gradient F(X[k]) || < gtol'
        #p.iterfcn()







#from numpy import asfarray, argmax, sign, inf, log10
from openopt.kernel.baseSolver import baseSolver
from numpy import isfinite, array, asfarray, ravel #asfarray,  inf,  atleast_1d
from pswarm_py import pswarm as PSWARM
from openopt.kernel.setDefaultIterFuncs import SMALL_DELTA_X,  SMALL_DELTA_F

#from openopt.kernel.setDefaultIterFuncs import SMALL_DELTA_X,  SMALL_DELTA_F

class pswarm(baseSolver):
    __name__ = 'pswarm'
    __license__ = "LGPL"
    __authors__ = 'A. I. F. Vaz (http://www.norg.uminho.pt/aivaz), connected to OO by Dmitrey'
    __alg__ = "A. I. F. Vaz and L. N. Vicente, A particle swarm pattern search method for bound constrained global optimization, Journal of Global Optimization, 39 (2007) 197-219. The algorithm combines pattern search and particle swarm. Basically, it applies a directional direct search in the poll step (coordinate search in the pure simple bounds case) and particle swarm in the search step."
    iterfcnConnected = True
    __homepage__ = 'http://www.norg.uminho.pt/aivaz/pswarm/'

    __info__ = "parameters: social (default = 0.5), cognitial (0.5), fweight (0.4), iweight (0.9), size (42), tol (1e-5), ddelta (0.5), idelta (2.0). Can handle constraints lb <= x <= ub (values beyond 1e20 are treated as 1e20), A x <= b. Documentation says pswarm is capable of using parallel calculations (via MPI) but I don't know is it relevant to Python API."
    __optionalDataThatCanBeHandled__ = ['lb', 'ub', 'A', 'b']
    __isIterPointAlwaysFeasible__ = lambda self, p: p.__isNoMoreThanBoxBounded__()

    social = 0.5
    cognitial = 0.5
    fweight = 0.4
    iweight = 0.9
    size = 42
    tol = 1e-5
    ddelta = 0.5
    idelta = 2.0

    def __init__(self):pass
    def __solver__(self, p):

        #if not p.__isFiniteBoxBounded__(): p.err('this solver requires finite lb, ub: lb <= x <= ub')
        
        p.kernelIterFuncs.pop(SMALL_DELTA_X, None)
        p.kernelIterFuncs.pop(SMALL_DELTA_F, None)
                
        lb, ub = p.lb, p.ub
        lb[lb < -1e20] = -1e20
        ub[ub > 1e20] = 1e20

        def f(x):
            if len(x) == 1:
                y = ravel(x)
            else:
                y = asfarray(x)
            return array(p.f(y), 'float')

        Problem = {
            'Variables':  p.n,
            'objf': f,
            'lb': lb.tolist(),
            'ub': ub.tolist(),
            }

        if any(isfinite(p.b)):
               Problem['A'] = p.A.tolist()
               Problem['b'] = p.b.tolist()
        if hasattr(p,'x0') and p.x0 is not None:
               Problem['x0'] = p.x0.tolist()

        def pswarm_iterfcn(it, leader, fx, x):
            p.xk = array(x)
            p.fk = array(fx)
            #print x, fx
            p.iterfcn()
            if p.istop != 0:
                p.debugmsg('istop:'+str(p.istop))
                return -1.0
            else: return 1.0

        Options = {
            'maxf': 2*p.maxFunEvals, # to provide correct istop triggered  from OO kernel
            'maxit': p.maxIter+15, # to provide correct istop triggered  from OO kernel
            'social': self.social,
            'cognitial': self.cognitial,
            'fweight': self.fweight,
            'iweight': self.iweight,
            'size': self.size,
            'tol': self.tol,
            'ddelta': self.ddelta,
            'idelta': self.idelta,
            'iprint': 1,
            'outputfcn': pswarm_iterfcn,
            'vectorized': 1
            }

        result = PSWARM(Problem,Options)

        #print 'results:', xf, ff
        p.xf, p.ff = result['x'], result['f']
        if p.istop == 0: p.istop = 1000
        #if p.istop == 0 : p.istop = 1000





from baseProblem import NonLinProblem


class ODE(NonLinProblem):
    probType = 'ODE'
    goal = 'solution'
    allowedGoals = ['solution']
    showGoal = False
    _optionalData = []
    FuncDesignerSign = 'equations'
    expectedArgs = ['equations', 'startPoint', 'times']
    ftol = None
    reltol = 1.49012e-8
    abstol = 1.49012e-8
    def __init__(self, *args, **kwargs):
        NonLinProblem.__init__(self, *args, **kwargs)
        domain= args[1]
        self.x0 = domain
#        if any(diff(times) < 0): self.err('''
#        currently required ODE times should be sorted 
#        in ascending order, other cases are unimplemented yet
#        ''')
        
        #self.constraints = [timeVariable > times[0], timeVariable < times[-1]]
        

    def objFunc(self, x):
        return 0
